from collections import OrderedDict
import pytest

from conan.api.model import Remote
from conan.internal.graph.graph_error import GraphConflictError, GraphMissingError
from conan.test.assets.genconanfile import GenConanfile
from test.integration.graph.core.graph_manager_base import GraphManagerTest
from conan.test.utils.tools import TestClient, TestServer, NO_SETTINGS_PACKAGE_ID


class TestVersionRanges(GraphManagerTest):

    def test_transitive(self):
        # app -> libb[>0.1]
        self.recipe_cache("libb/0.1")
        self.recipe_cache("libb/0.2")
        consumer = self.recipe_consumer("app/0.1", ["libb/[>=0.0]"])
        deps_graph = self.build_consumer(consumer)

        assert 2 == len(deps_graph.nodes)
        app = deps_graph.root
        libb = app.edges[0].dst

        self._check_node(libb, "libb/0.2#123", dependents=[app])
        self._check_node(app, "app/0.1", deps=[libb])

    def test_transitive_local_conditions(self):
        for v in ["0.1", "0.2", "0.3", "1.1", "1.1.2", "1.2.1", "2.1", "2.2.1"]:
            self.recipe_cache(f"libb/{v}")

        for expr, solution in [(">0.0", "2.2.1"),
                               (">0.1 <1", "0.3"),
                               (">0.1 <1||2.1", "2.1"),
                               ("", "2.2.1"),
                               ("~0", "0.3"),
                               ("~1", "1.2.1"),
                               ("~1.1", "1.1.2"),
                               ("~2", "2.2.1"),
                               ("~2.1", "2.1"),
                               ]:
            consumer = self.recipe_consumer("app/0.1", [f"libb/[{expr}]"])
            deps_graph = self.build_consumer(consumer)
            assert 2 == len(deps_graph.nodes)
            app = deps_graph.root
            libb = app.edges[0].dst

            self._check_node(libb, f"libb/{solution}#123", dependents=[app])
            self._check_node(app, "app/0.1", deps=[libb])

    def test_missing(self):
        # app -> libb[>0.1] (missing)
        consumer = self.recipe_consumer("app/0.1", ["libb/[>=0.0]"])

        deps_graph = self.build_consumer(consumer, install=False)

        assert type(deps_graph.error) == GraphMissingError

        assert 1 == len(deps_graph.nodes)
        app = deps_graph.root
        self._check_node(app, "app/0.1", deps=[])

    def test_userchannel_no_match(self):
        # app -> libb[>0.1] (missing)
        self.recipe_cache("libb/0.1@user/channel")
        consumer = self.recipe_consumer("app/0.1", ["libb/[>=0.0]"])

        deps_graph = self.build_consumer(consumer, install=False)

        assert type(deps_graph.error) == GraphMissingError

        assert 1 == len(deps_graph.nodes)
        app = deps_graph.root

        self._check_node(app, "app/0.1", deps=[])

    def test_required_userchannel_no_match(self):
        # app -> libb[>0.1] (missing)
        self.recipe_cache("libb/0.1")
        consumer = self.recipe_consumer("app/0.1", ["libb/[>=0.0]@user/channel"])

        deps_graph = self.build_consumer(consumer, install=False)

        assert type(deps_graph.error) == GraphMissingError

        assert 1 == len(deps_graph.nodes)
        app = deps_graph.root

        self._check_node(app, "app/0.1", deps=[])

    def test_transitive_out_range(self):
        # app -> libb[>0.1] (missing)
        self.recipe_cache("libb/0.1")
        consumer = self.recipe_consumer("app/0.1", ["libb/[>1.0]"])

        deps_graph = self.build_consumer(consumer, install=False)

        assert type(deps_graph.error) == GraphMissingError

        assert 1 == len(deps_graph.nodes)
        app = deps_graph.root

        self._check_node(app, "app/0.1", deps=[])


class TestVersionRangesDiamond(GraphManagerTest):
    def test_transitive(self):
        # app -> libb/0.1 -(range >0)-> liba/0.2
        #   \ -> libc/0.1 -(range <1)---/
        self.recipe_cache("liba/0.1")
        self.recipe_cache("liba/0.2")
        self.recipe_cache("libb/0.1", ["liba/[>=0.0]"])
        self.recipe_cache("libc/0.1", ["liba/[<1.0]"])
        consumer = self.recipe_consumer("app/0.1", ["libb/0.1", "libc/0.1"])
        deps_graph = self.build_consumer(consumer)

        assert 4 == len(deps_graph.nodes)
        app = deps_graph.root
        libb = app.edges[0].dst
        libc = app.edges[1].dst
        liba = libb.edges[0].dst

        self._check_node(liba, "liba/0.2#123", dependents=[libb, libc], deps=[])
        self._check_node(libb, "libb/0.1#123", dependents=[app], deps=[liba])
        self._check_node(libc, "libc/0.1#123", dependents=[app], deps=[liba])
        self._check_node(app, "app/0.1", deps=[libb, libc])

    def test_transitive_interval(self):
        # app -> libb/0.1 -(range >0 <0.3)-> liba/0.2
        #   \ -> libc/0.1 -(range <1)--------/
        self.recipe_cache("liba/0.1")
        self.recipe_cache("liba/0.2")
        self.recipe_cache("liba/0.3")
        self.recipe_cache("libb/0.1", ["liba/[>=0.0 <0.3]"])
        self.recipe_cache("libc/0.1", ["liba/[<1.0]"])
        consumer = self.recipe_consumer("app/0.1", ["libb/0.1", "libc/0.1"])
        deps_graph = self.build_consumer(consumer)

        assert 4 == len(deps_graph.nodes)
        app = deps_graph.root
        libb = app.edges[0].dst
        libc = app.edges[1].dst
        liba = libb.edges[0].dst

        self._check_node(liba, "liba/0.2#123", dependents=[libb, libc], deps=[])
        self._check_node(libb, "libb/0.1#123", dependents=[app], deps=[liba])
        self._check_node(libc, "libc/0.1#123", dependents=[app], deps=[liba])
        self._check_node(app, "app/0.1", deps=[libb, libc])

    def test_transitive_fixed(self):
        # app -> libb/0.1 --------> liba/0.1
        #   \ -> libc/0.1 -(range <1)---/
        self.recipe_cache("liba/0.1")
        self.recipe_cache("liba/0.2")
        self.recipe_cache("libb/0.1", ["liba/0.1"])
        self.recipe_cache("libc/0.1", ["liba/[<1.0]"])
        consumer = self.recipe_consumer("app/0.1", ["libb/0.1", "libc/0.1"])
        deps_graph = self.build_consumer(consumer)

        assert 4 == len(deps_graph.nodes)
        app = deps_graph.root
        libb = app.edges[0].dst
        libc = app.edges[1].dst
        liba = libb.edges[0].dst

        self._check_node(liba, "liba/0.1#123", dependents=[libb, libc], deps=[])
        self._check_node(libb, "libb/0.1#123", dependents=[app], deps=[liba])
        self._check_node(libc, "libc/0.1#123", dependents=[app], deps=[liba])
        self._check_node(app, "app/0.1", deps=[libb, libc])

    def test_transitive_conflict(self):
        # app -> libb/0.1 -(range >0)-> liba/0.2
        #   \ -> libc/0.1 -(range >1)---/
        self.recipe_cache("liba/0.2")
        self.recipe_cache("libb/0.1", ["liba/[>=0.0]"])
        self.recipe_cache("libc/0.1", ["liba/[>1.0]"])
        consumer = self.recipe_consumer("app/0.1", ["libb/0.1", "libc/0.1"])
        deps_graph = self.build_consumer(consumer, install=False)

        assert type(deps_graph.error) == GraphConflictError

        assert 4 == len(deps_graph.nodes)
        app = deps_graph.root
        app.enabled_remotes = [Remote("foo", None)]
        libb = app.edges[0].dst
        libc = app.edges[1].dst
        liba = libb.edges[0].dst

        self._check_node(libb, "libb/0.1#123", dependents=[app], deps=[liba])
        self._check_node(libb, "libb/0.1#123", dependents=[app], deps=[liba])
        self._check_node(libc, "libc/0.1#123", dependents=[app], deps=[])
        self._check_node(app, "app/0.1", deps=[libb, libc])

    def test_transitive_fixed_conflict(self):
        # app -> libb/0.1 ---------> liba/0.2
        #   \ -> libc/0.1 -(range >1)---/
        self.recipe_cache("liba/0.2")
        self.recipe_cache("libb/0.1", ["liba/[>=0.0]"])
        self.recipe_cache("libc/0.1", ["liba/[>1.0]"])
        consumer = self.recipe_consumer("app/0.1", ["libb/0.1", "libc/0.1"])
        deps_graph = self.build_consumer(consumer, install=False)

        assert type(deps_graph.error) == GraphConflictError

        assert 4 == len(deps_graph.nodes)
        app = deps_graph.root
        libb = app.edges[0].dst
        libc = app.edges[1].dst
        liba = libb.edges[0].dst

        self._check_node(libb, "libb/0.1#123", dependents=[app], deps=[liba])
        self._check_node(libb, "libb/0.1#123", dependents=[app], deps=[liba])
        self._check_node(libc, "libc/0.1#123", dependents=[app], deps=[])
        self._check_node(app, "app/0.1", deps=[libb, libc])


class TestVersionRangesOverridesDiamond(GraphManagerTest):
    def test_transitive(self):
        # app -> libb/0.1 -(range >0)-> liba/0.2
        #   \ ---------------------------/
        self.recipe_cache("liba/0.1")
        self.recipe_cache("liba/0.2")
        self.recipe_cache("libb/0.1", ["liba/[>=0.0]"])
        consumer = self.recipe_consumer("app/0.1", ["libb/0.1", "liba/0.2"])
        deps_graph = self.build_consumer(consumer)

        assert 3 == len(deps_graph.nodes)
        app = deps_graph.root
        libb = app.edges[0].dst
        liba = libb.edges[0].dst

        self._check_node(liba, "liba/0.2#123", dependents=[libb, app], deps=[])
        self._check_node(libb, "libb/0.1#123", dependents=[app], deps=[liba])
        self._check_node(app, "app/0.1", deps=[libb, liba])

    def test_transitive_overriden(self):
        # app -> libb/0.1 -(range >0)-> liba/0.1
        #   \ ---------liba/0.1-------------/
        self.recipe_cache("liba/0.1")
        self.recipe_cache("liba/0.2")
        self.recipe_cache("libb/0.1", ["liba/[>=0.0]"])
        consumer = self.recipe_consumer("app/0.1", ["libb/0.1", "liba/0.1"])
        deps_graph = self.build_consumer(consumer)

        assert 3 == len(deps_graph.nodes)
        app = deps_graph.root
        libb = app.edges[0].dst
        liba = libb.edges[0].dst

        self._check_node(liba, "liba/0.1#123", dependents=[libb, app], deps=[])
        self._check_node(libb, "libb/0.1#123", dependents=[app], deps=[liba])
        self._check_node(app, "app/0.1", deps=[libb, liba])

    def test_transitive_fixed(self):
        # app ---> libb/0.1 -----------> liba/0.1
        #   \ --------(range<1)----------/
        self.recipe_cache("liba/0.1")
        self.recipe_cache("liba/0.2")
        self.recipe_cache("libb/0.1", ["liba/0.1"])
        consumer = self.recipe_consumer("app/0.1", ["libb/0.1", "liba/[<1.0]"])
        deps_graph = self.build_consumer(consumer)

        assert 3 == len(deps_graph.nodes)
        app = deps_graph.root
        libb = app.edges[0].dst
        liba = libb.edges[0].dst

        self._check_node(liba, "liba/0.1#123", dependents=[libb, app], deps=[])
        self._check_node(libb, "libb/0.1#123", dependents=[app], deps=[liba])
        self._check_node(app, "app/0.1", deps=[libb, liba])

    def test_transitive_fixed_conflict(self):
        # app ---> libb/0.1 -----------> liba/0.1
        #   \ --------(range>1)----------/
        self.recipe_cache("liba/0.1")
        self.recipe_cache("liba/1.2")
        self.recipe_cache("libb/0.1", ["liba/0.1"])
        consumer = self.recipe_consumer("app/0.1", ["libb/0.1", "liba/[>1.0]"])
        deps_graph = self.build_consumer(consumer, install=False)

        assert type(deps_graph.error) == GraphConflictError

        assert 2 == len(deps_graph.nodes)
        app = deps_graph.root
        libb = app.edges[0].dst

    def test_transitive_fixed_conflict_forced(self):
        # app ---> libb/0.1 -----------> liba/1.2
        #   \ --------(range>1)----------/
        self.recipe_cache("liba/0.1")
        self.recipe_cache("liba/1.2")
        self.recipe_cache("libb/0.1", ["liba/0.1"])
        consumer = self.consumer_conanfile(GenConanfile("app", "0.1").with_require("libb/0.1")
                                           .with_requirement("liba/[>1.0]", force=True))
        deps_graph = self.build_consumer(consumer, install=False)

        assert 3 == len(deps_graph.nodes)
        app = deps_graph.root
        libb = app.edges[0].dst
        liba = libb.edges[0].dst

        self._check_node(liba, "liba/1.2#123", dependents=[libb, app], deps=[])
        self._check_node(libb, "libb/0.1#123", dependents=[app], deps=[liba])
        self._check_node(app, "app/0.1", deps=[libb, liba])

    def test_two_ranges_overriden(self):
        # app -> libb/0.1 -(range >0)-> liba/0.1
        #   \ ---------liba/[<0.3>]-------------/
        self.recipe_cache("liba/0.1")
        self.recipe_cache("liba/0.2")
        self.recipe_cache("liba/0.3")
        self.recipe_cache("libb/0.1", ["liba/[>=0.0]"])
        consumer = self.consumer_conanfile(GenConanfile("app", "0.1").with_require("libb/0.1")
                                           .with_requirement("liba/[<0.4]"))
        deps_graph = self.build_consumer(consumer)

        assert 3 == len(deps_graph.nodes)
        app = deps_graph.root
        libb = app.edges[0].dst
        liba = libb.edges[0].dst

        self._check_node(liba, "liba/0.3#123", dependents=[libb, app], deps=[])
        self._check_node(libb, "libb/0.1#123", dependents=[app], deps=[liba])
        self._check_node(app, "app/0.1", deps=[libb, liba])

    def test_two_ranges_overriden_no_conflict(self):
        # app -> libb/0.1 -(range >0)-> liba/0.1
        #   \ ---------liba/[<0.3>]-------------/
        # Conan learned to solve this conflict in 2.0.14
        self.recipe_cache("liba/0.1")
        self.recipe_cache("liba/0.2")
        self.recipe_cache("liba/0.3")
        self.recipe_cache("libb/0.1", ["liba/[>=0.0]"])
        consumer = self.consumer_conanfile(GenConanfile("app", "0.1").with_require("libb/0.1")
                                           .with_requirement("liba/[<0.3]"))
        deps_graph = self.build_consumer(consumer, install=False)

        # This is no longer a conflict, and Conan knows that liba/2.0 is a valid joint solution

        assert 3 == len(deps_graph.nodes)
        app = deps_graph.root
        libb = app.edges[0].dst
        liba = libb.edges[0].dst

        self._check_node(liba, "liba/0.2#123", dependents=[libb, app], deps=[])
        self._check_node(libb, "libb/0.1#123", dependents=[app], deps=[liba])
        self._check_node(app, "app/0.1", deps=[libb, liba])


def test_mixed_user_channel():
    # https://github.com/conan-io/conan/issues/7846
    t = TestClient(default_server_user=True, light=True)
    t.save({"conanfile.py": GenConanfile()})
    t.run("create . --name=pkg --version=1.0")
    t.run("create . --name=pkg --version=1.1")
    t.run("create . --name=pkg --version=2.0")
    t.run("create . --name=pkg --version=1.0 --user=user --channel=testing")
    t.run("create . --name=pkg --version=1.1 --user=user --channel=testing")
    t.run("create . --name=pkg --version=2.0 --user=user --channel=testing")
    t.run("upload * --confirm -r default")
    t.run("remove * -c")

    t.run('install --requires="pkg/[>0 <2]@"')
    t.assert_listed_require({"pkg/1.1": "Downloaded (default)"})
    t.run('install --requires="pkg/[>0 <2]@user/testing"')
    t.assert_listed_require({"pkg/1.1@user/testing": "Downloaded (default)"})


def test_remote_version_ranges():
    t = TestClient(default_server_user=True, light=True)
    t.save({"conanfile.py": GenConanfile()})
    for v in ["0.1", "0.2", "0.3", "1.1", "1.1.2", "1.2.1", "2.1", "2.2.1"]:
        t.run(f"create . --name=dep --version={v}")
    t.run("upload * --confirm -r default")
    # TODO: Deprecate the comma separator for expressions
    for expr, solution in [(">0.0", "2.2.1"),
                           (">0.1 <1", "0.3"),
                           (">0.1 <1||2.1", "2.1"),
                           ("", "2.2.1"),
                           ("~0", "0.3"),
                           ("~1", "1.2.1"),
                           ("~1.1", "1.1.2"),
                           ("~2", "2.2.1"),
                           ("~2.1", "2.1"),
                           ]:
        t.run("remove * -c")
        t.save({"conanfile.py": GenConanfile().with_requires(f"dep/[{expr}]")})
        t.run("install .")
        assert str(t.out).count("Not found in local cache, looking in remotes") == 1
        t.assert_listed_binary({f"dep/{solution}": ("da39a3ee5e6b4b0d3255bfef95601890afd80709",
                                                    "Download (default)")})


def test_different_user_channel_resolved_correctly():
    server1 = TestServer()
    server2 = TestServer()
    servers = OrderedDict([("server1", server1), ("server2", server2)])

    client = TestClient(servers=servers, inputs=2*["admin", "password"], light=True)
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . --name=lib --version=1.0 --user=conan --channel=stable")
    client.run("create . --name=lib --version=1.0 --user=conan --channel=testing")
    client.run("upload lib/1.0@conan/stable -r=server1")
    client.run("upload lib/1.0@conan/testing -r=server2")

    client2 = TestClient(servers=servers, light=True)
    client2.run("install --requires=lib/[>=1.0]@conan/testing")
    assert f"lib/1.0@conan/testing: Retrieving package {NO_SETTINGS_PACKAGE_ID} " \
           f"from remote 'server2' " in client2.out


def test_unknown_options():
    c = TestClient(light=True)
    c.save({"conanfile.py": GenConanfile("lib", "2.0")})
    c.run("create .")

    c.run("graph info --requires=lib/[>1.2,<1.4]", assert_error=True)
    assert '"<1.4" in version range ">1.2,<1.4" is not a valid option' in c.out

    c.run("graph info --requires=lib/[>1.2,unknown_conf]")
    assert 'WARN: Unrecognized version range option "unknown_conf" in ">1.2,unknown_conf"' in c.out


@pytest.mark.parametrize("version_range,should_warn", [
    [">=0.1, include_prereleases", False],
    [">=0.1, include_prerelease=True", True],
    [">=0.1, include_prerelease=False", True]
])
def test_bad_options_syntax(version_range, should_warn):
    """We don't error out on bad options, maybe we should,
    but for now this test ensures we don't change it without realizing"""
    tc = TestClient(light=True)
    tc.save({"lib/conanfile.py": GenConanfile("lib", "1.0"),
             "app/conanfile.py": GenConanfile("app", "1.0").with_requires(f"lib/[{version_range}]")})
    tc.run("export lib")
    tc.run("graph info app/conanfile.py")
    if should_warn:
        assert "its presence unconditionally enables prereleases" in tc.out
    else:
        assert "its presence unconditionally enables prereleases" not in tc.out


def test_empty_version_ranger():
    tc = TestClient(light=True)
    tc.save({"lib/conanfile.py": GenConanfile("lib", "1.0"),
             "app/conanfile.py": GenConanfile("app", "1.0").with_requires("lib/[]")})
    tc.run("export lib")
    tc.run("graph info app")
    assert "lib/[]: lib/1.0" in tc.out
    assert "Empty version range usage is discouraged" in tc.out



from unittest.mock import Mock

from conan.internal.graph.graph import CONTEXT_HOST
from conan.internal.graph.graph_builder import DepsGraph, Node
from conan.internal.model.conan_file import ConanFile
from conan.api.model import RecipeReference


class TestDepsGraph:

    def test_node(self):
        """ nodes are different even if contain same values,
        so they can be repeated if necessary in the graph (common
        static libraries)
        """
        ref1 = RecipeReference.loads("hello/0.1@user/stable")
        ref2 = RecipeReference.loads("hello/0.1@user/stable")

        conanfile1 = ConanFile(None)
        conanfile2 = ConanFile(None)
        n1 = Node(ref1, conanfile1, context=CONTEXT_HOST)
        n2 = Node(ref2, conanfile2, context=CONTEXT_HOST)

        assert n1 != n2

    def test_basic_levels(self):
        ref1 = RecipeReference.loads("hello/1.0@user/stable")
        ref2 = RecipeReference.loads("hello/2.0@user/stable")
        ref3 = RecipeReference.loads("hello/3.0@user/stable")

        deps = DepsGraph()
        n1 = Node(ref1, Mock(), context=CONTEXT_HOST)
        n2 = Node(ref2, Mock(), context=CONTEXT_HOST)
        n3 = Node(ref3, Mock(), context=CONTEXT_HOST)
        deps.add_node(n1)
        deps.add_node(n2)
        deps.add_node(n3)
        deps.add_edge(n1, n2, None)
        deps.add_edge(n2, n3, None)
        assert [[n3], [n2], [n1]] == deps.by_levels()

    def test_multi_levels(self):
        ref1 = RecipeReference.loads("hello/1.0@user/stable")
        ref2 = RecipeReference.loads("hello/2.0@user/stable")
        ref31 = RecipeReference.loads("hello/31.0@user/stable")
        ref32 = RecipeReference.loads("hello/32.0@user/stable")

        deps = DepsGraph()
        n1 = Node(ref1, Mock(), context=CONTEXT_HOST)
        n2 = Node(ref2, Mock(), context=CONTEXT_HOST)
        n31 = Node(ref31, Mock(), context=CONTEXT_HOST)
        n32 = Node(ref32, Mock(), context=CONTEXT_HOST)
        deps.add_node(n1)
        deps.add_node(n2)
        deps.add_node(n32)
        deps.add_node(n31)
        deps.add_edge(n1, n2, None)
        deps.add_edge(n2, n31, None)
        deps.add_edge(n2, n32, None)
        assert [[n31, n32], [n2], [n1]] == deps.by_levels()

    def test_multi_levels_2(self):

        ref1 = RecipeReference.loads("hello/1.0@user/stable")
        ref2 = RecipeReference.loads("hello/2.0@user/stable")
        ref5 = RecipeReference.loads("hello/5.0@user/stable")
        ref31 = RecipeReference.loads("hello/31.0@user/stable")
        ref32 = RecipeReference.loads("hello/32.0@user/stable")

        deps = DepsGraph()
        n1 = Node(ref1, Mock(), context=CONTEXT_HOST)
        n2 = Node(ref2, Mock(), context=CONTEXT_HOST)
        n5 = Node(ref5, Mock(), context=CONTEXT_HOST)
        n31 = Node(ref31, Mock(), context=CONTEXT_HOST)
        n32 = Node(ref32, Mock(), context=CONTEXT_HOST)
        deps.add_node(n1)
        deps.add_node(n5)
        deps.add_node(n2)
        deps.add_node(n32)
        deps.add_node(n31)
        deps.add_edge(n1, n2, None)
        deps.add_edge(n1, n5, None)
        deps.add_edge(n2, n31, None)
        deps.add_edge(n2, n32, None)
        assert [[n31, n32, n5], [n2], [n1]] == deps.by_levels()

    def test_multi_levels_3(self):

        ref1 = RecipeReference.loads("hello/1.0@user/stable")
        ref2 = RecipeReference.loads("hello/2.0@user/stable")
        ref5 = RecipeReference.loads("hello/5.0@user/stable")
        ref31 = RecipeReference.loads("hello/31.0@user/stable")
        ref32 = RecipeReference.loads("hello/32.0@user/stable")

        deps = DepsGraph()
        n1 = Node(ref1, Mock(), context=CONTEXT_HOST)
        n2 = Node(ref2, Mock(), context=CONTEXT_HOST)
        n5 = Node(ref5, Mock(), context=CONTEXT_HOST)
        n31 = Node(ref31, Mock(), context=CONTEXT_HOST)
        n32 = Node(ref32, Mock(), context=CONTEXT_HOST)
        deps.add_node(n1)
        deps.add_node(n5)
        deps.add_node(n2)
        deps.add_node(n32)
        deps.add_node(n31)
        deps.add_edge(n1, n2, None)
        deps.add_edge(n1, n5, None)
        deps.add_edge(n2, n31, None)
        deps.add_edge(n2, n32, None)
        deps.add_edge(n32, n5, None)
        assert [[n31, n5], [n32], [n2], [n1]] == deps.by_levels()



import json
import os
import textwrap
from unittest import mock
from unittest.mock import MagicMock

import pytest

from conan import __version__
from conan.internal.model.conf import ConfDefinition
from conan.internal.rest.conan_requester import ConanRequester
from conan.test.utils.file_server import TestFileServer
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient
from conan.internal.util.files import save


class TestAuthSourcePlugin:
    @pytest.fixture
    def setup_test_client(self):
        client = TestClient(default_server_user=True, light=True)
        file_server = TestFileServer()
        client.servers["file_server"] = file_server
        client.save_home({"global.conf": "tools.files.download:retry=0"})

        save(os.path.join(file_server.store, "myfile.txt"), "Bye, world!")

        conanfile = textwrap.dedent(f"""
           from conan import ConanFile
           from conan.tools.files import download
           class Pkg2(ConanFile):
               name = "pkg"
               version = "1.0"
               def source(self):
                   download(self, "{file_server.fake_url}/basic-auth/myfile.txt", "myfile.txt")
           """)
        client.save({"conanfile.py": conanfile})

        return client, file_server.fake_url

    def test_error_source_plugin(self, setup_test_client):
        """ Test when the plugin fails, we want a clear message and a helpful trace
        """
        c, url = setup_test_client
        auth_plugin = textwrap.dedent("""\
            def auth_source_plugin(url):
                raise Exception("Test Error")
            """)
        c.save_home({"extensions/plugins/auth_source.py": auth_plugin})
        c.run("source conanfile.py", assert_error=True)
        assert "Test Error" in c.out

    @pytest.mark.parametrize("password", ["password", "bad-password"])
    def test_auth_source_plugin_direct_credentials(self, password, setup_test_client):
        """ Test when the plugin give a correct and wrong password, we want a message about
        the success or fail in login
        """
        should_fail = password == "bad-password"
        c, url = setup_test_client
        auth_plugin = textwrap.dedent(f"""\
            def auth_source_plugin(url):
                return {json.dumps({'user': 'user', 'password': password})}
            """)
        c.save_home({"extensions/plugins/auth_source.py": auth_plugin})
        c.run("source conanfile.py", assert_error=should_fail)
        if should_fail:
            assert "AuthenticationException" in c.out
        else:
            assert os.path.exists(os.path.join(c.current_folder, "myfile.txt"))

    def test_auth_source_plugin_fallback(self, setup_test_client):
        """ Test when the plugin do not give any user or password, we want the code to continue with
        the rest of the input methods
        """
        c, url = setup_test_client
        auth_plugin = textwrap.dedent("""\
                def auth_source_plugin(url):
                    return None
                """)
        c.save_home({"extensions/plugins/auth_source.py": auth_plugin})
        source_credentials = json.dumps({"credentials": [{"url": url, "token": "password"}]})
        c.save_home({"source_credentials.json": source_credentials})
        c.run("source conanfile.py")
        # As the auth plugin is not returning any password the code is falling back to the rest of
        # the input methods in this case provided by source_credentials.json.
        assert os.path.exists(os.path.join(c.current_folder, "myfile.txt"))

    def test_source_auth_plugin_headers(self):
        auth_plugin = textwrap.dedent(f"""\
            def auth_source_plugin(url):
                return {json.dumps({'headers': {"myheader": "myvalue"}})}
            """)
        cache_folder = temp_folder()
        save(os.path.join(cache_folder, "extensions/plugins/auth_source.py"), auth_plugin)

        mock_http_requester = MagicMock()
        with mock.patch("conan.internal.rest.conan_requester.requests", mock_http_requester):
            requester = ConanRequester(ConfDefinition(), cache_folder)
            requester.get(url="aaa", source_credentials=True)
            headers = requester._http_requester.get.call_args[1]["headers"]
            assert headers["myheader"] == "myvalue"
            assert f"Conan/{__version__}" in headers["User-Agent"]

    def test_source_credentials_headers(self):
        cache_folder = temp_folder()
        source_credentials = json.dumps({"credentials": [{"url": "aaa", "token": "mytok",
                                                          "headers": {"myheader2": "myvalue2"}}]})
        save(os.path.join(cache_folder, "source_credentials.json"), source_credentials)

        mock_http_requester = MagicMock()
        with mock.patch("conan.internal.rest.conan_requester.requests", mock_http_requester):
            requester = ConanRequester(ConfDefinition(), cache_folder)
            requester.get(url="aaa", source_credentials=True)
            headers = requester._http_requester.get.call_args[1]["headers"]
            assert headers["myheader2"] == "myvalue2"
            assert f"Conan/{__version__}" in headers["User-Agent"]



import os
import textwrap
from collections import OrderedDict
from unittest import mock

from conan.api.model import RecipeReference
from conan.test.utils.tools import TestClient, TestServer, NO_SETTINGS_PACKAGE_ID, GenConanfile
from conan.internal.util.files import load


def test_download_with_sources():
    client = TestClient(default_server_user=True)
    client.save({"conanfile.py": GenConanfile("pkg", "0.1").with_exports_sources("*"),
                 "file.h": "myfile.h",
                 "otherfile.cpp": "C++code"})
    client.run("export . --user=lasote --channel=stable")

    ref = RecipeReference.loads("pkg/0.1@lasote/stable")
    client.run("upload pkg/0.1@lasote/stable -r default")
    client.run("remove pkg/0.1@lasote/stable -c")

    client.run("download pkg/0.1@lasote/stable -r default")
    assert "Downloading 'pkg/0.1@lasote/stable' sources" in client.out
    source = client.get_latest_ref_layout(ref).export_sources()
    assert "myfile.h" == load(os.path.join(source, "file.h"))
    assert "C++code" == load(os.path.join(source, "otherfile.cpp"))


def test_no_user_channel():
    # https://github.com/conan-io/conan/issues/6009
    client = TestClient(default_server_user=True)
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . --name=pkg --version=1.0")
    client.run("upload * --confirm -r default")
    client.run("remove * -c")

    client.run("download pkg/1.0:{} -r default".format(NO_SETTINGS_PACKAGE_ID))
    assert f"Downloading package 'pkg/1.0#4d670581ccb765839f2239cc8dff8fbd:{NO_SETTINGS_PACKAGE_ID}" in client.out

    # All
    client.run("remove * -c")
    client.run("download pkg/1.0#*:* -r default")
    assert f"Downloading package 'pkg/1.0#4d670581ccb765839f2239cc8dff8fbd:{NO_SETTINGS_PACKAGE_ID}" in client.out


def test_download_with_python_requires():
    """ In the past,
    when having a python_require in a different repo, it cannot be ``conan download``
    as the download runs from a single repo.

    Now, from https://github.com/conan-io/conan/issues/14260, "conan download" doesn't
    really need to load conanfile, so it doesn't fail because of this.
    """
    # https://github.com/conan-io/conan/issues/9548
    servers = OrderedDict([("tools", TestServer()),
                           ("pkgs", TestServer())])
    c = TestClient(servers=servers, inputs=["admin", "password", "admin", "password"])

    c.save({"tool/conanfile.py": GenConanfile("tool", "0.1"),
            "pkg/conanfile.py": GenConanfile("pkg", "0.1").with_python_requires("tool/0.1")})
    c.run("export tool")
    c.run("create pkg")
    c.run("upload tool* -r tools -c")
    c.run("upload pkg* -r pkgs -c")
    c.run("remove * -c")

    c.run("install --requires=pkg/0.1 -r pkgs -r tools")
    assert "Downloading" in c.out
    c.run("remove * -c")

    c.run("download pkg/0.1 -r pkgs")
    assert "pkg/0.1: Downloaded package revision" in c.out


def test_download_verify_ssl_conf():
    client = TestClient()

    client.save({"conanfile.py": textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import download

        class Pkg(ConanFile):
            name = "pkg"
            version = "1.0"

            def source(self):
                download(self, "http://verify.true", "", verify=True)
                download(self, "http://verify.false", "", verify=False)
        """)})

    did_verify = {}

    def custom_download(this, url, filepath, *args, **kwargs):
        did_verify[url] = args[2]

    with mock.patch("conan.internal.rest.file_downloader.FileDownloader.download",
                    custom_download):
        client.run("create . -c tools.files.download:verify=True")
        assert did_verify["http://verify.true"]
        assert did_verify["http://verify.false"]

        did_verify.clear()
        client.run("remove pkg/1.0 -c")

        client.run("create . -c tools.files.download:verify=False")
        assert not did_verify["http://verify.true"]
        assert not did_verify["http://verify.false"]

        did_verify.clear()
        client.run("remove pkg/1.0 -c")

        client.run("create .")
        assert did_verify["http://verify.true"]
        assert not did_verify["http://verify.false"]


def test_download_list_only_recipe():
    c = TestClient(default_server_user=True)
    c.save({"conanfile.py": GenConanfile("liba", "0.1")})
    c.run("create .")
    c.run("upload * -r=default -c")
    c.run("remove * -c")
    c.run("list *:* -r=default --format=json", redirect_stdout="pkgs.json")
    c.run("download --list=pkgs.json --only-recipe -r=default")
    assert "packages" not in c.out



import textwrap
from conan.test.assets.genconanfile import GenConanfile
import pytest

from conan.test.utils.tools import TestClient
from conan.tools.premake.toolchain import PremakeToolchain


def test_extra_flags_via_conf():
    profile = textwrap.dedent(
        """
        [settings]
        os=Linux
        arch=x86_64
        compiler=gcc
        compiler.version=9
        compiler.cppstd=17
        compiler.libcxx=libstdc++
        build_type=Release

        [conf]
        tools.build:cxxflags=["-flag1", "-flag2"]
        tools.build:cflags=["-flag3", "-flag4"]
        tools.build:sharedlinkflags+=["-flag5"]
        tools.build:exelinkflags+=["-flag6"]
        tools.build:defines=["define1=0"]
    """
    )
    tc = TestClient()
    tc.save(
        {
            "conanfile.txt": "[generators]\nPremakeToolchain",
            "profile": profile,
        }
    )

    tc.run("install . -pr:a=profile")
    content = tc.load(PremakeToolchain.filename)
    assert 'cppdialect "c++17"' in content

    assert (
        """
        filter { files { "**.c" } }
            buildoptions { "-flag3", "-flag4", "-m64" }
        filter {}
        """
        in content
    )

    assert (
        """
        filter { files { "**.cpp", "**.cxx", "**.cc" } }
            buildoptions { "-flag1", "-flag2", "-m64" }
        filter {}
        """
        in content
    )

    assert 'linkoptions { "-flag5", "-flag6", "-m64" }' in content
    assert 'defines { "define1=0", "_GLIBCXX_USE_CXX11_ABI=0" }' in content


def test_project_configuration():
    tc = TestClient(path_with_spaces=False)
    profile = textwrap.dedent(
        """
        [settings]
        arch=armv8
        build_type=Release
        compiler=apple-clang
        compiler.cppstd=gnu17
        compiler.cstd=gnu11
        compiler.libcxx=libc++
        compiler.version=17
        os=Macos
    """
    )
    conanfile = textwrap.dedent(
        """
        from conan import ConanFile
        from conan.tools.layout import basic_layout
        from conan.tools.premake import Premake, PremakeDeps, PremakeToolchain
        import os

        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            name = "pkg"
            version = "1.0"
            options = {"fPIC": [True, False]}
            default_options = {"fPIC": True}

            def layout(self):
                basic_layout(self, src_folder="src")

            def generate(self):
                tc = PremakeToolchain(self)
                tc.extra_defines = ["VALUE=2"]
                tc.extra_cflags = ["-Wextra"]
                tc.extra_cxxflags = ["-Wall", "-Wextra"]
                tc.extra_ldflags = ["-lm"]
                tc.project("main").extra_defines = ["TEST=False"]
                tc.project("test").disable = True
                tc.project("main").extra_cxxflags = ["-FS"]
                tc.generate()
    """
    )
    tc.save({"conanfile.py": conanfile, "profile": profile})
    tc.run("install . -pr:a profile")

    toolchain = tc.load("build-release/conan/conantoolchain.premake5.lua")

    assert 'pic "On"' in toolchain
    assert (
        """
        filter { files { "**.c" } }
            buildoptions { "-Wextra" }
        filter {}
        """
        in toolchain
    )
    assert (
        """
        filter { files { "**.cpp", "**.cxx", "**.cc" } }
            buildoptions { "-stdlib=libc++", "-Wall", "-Wextra" }
        filter {}
        """
        in toolchain
    )
    assert 'linkoptions { "-lm" }' in toolchain
    assert 'defines { "VALUE=2" }' in toolchain
    assert 'linkoptions { "-Wl,-rpath,@loader_path" }' in toolchain
    assert (
        textwrap.dedent(
            """
    project "main"
        -- CXX flags retrieved from CXXFLAGS environment, conan.conf(tools.build:cxxflags), extra_cxxflags and compiler settings
        filter { files { "**.cpp", "**.cxx", "**.cc" } }
            buildoptions { "-stdlib=libc++", "-FS" }
        filter {}
        -- Defines retrieved from DEFINES environment, conan.conf(tools.build:defines) and extra_defines
        defines { "TEST=False" }
        """
        )
        in toolchain
    )
    assert (
        textwrap.dedent(
            """
        project "test"
            kind "None"
        """
        )
        in toolchain
    )


@pytest.mark.parametrize(
    "threads, flags",
    [("posix", "-pthread"), ("wasm_workers", "-sWASM_WORKERS=1")],
)
def test_thread_flags(threads, flags):
    client = TestClient()
    profile = textwrap.dedent(f"""
        [settings]
        arch=wasm64
        build_type=Release
        compiler=emcc
        compiler.cppstd=17
        compiler.threads={threads}
        compiler.libcxx=libc++
        compiler.version=4.0.10
        os=Emscripten
        """)
    client.save(
        {
            "conanfile.py": GenConanfile("pkg", "1.0")
            .with_settings("os", "arch", "compiler", "build_type")
            .with_generator("PremakeToolchain"),
            "profile": profile,
        }
    )
    client.run("install . -pr=./profile")
    toolchain = client.load("conantoolchain.premake5.lua")
    assert (
        """
        filter { files { "**.c" } }
            buildoptions { "-sMEMORY64=1", "%s" }
        filter {}
        """
        % flags
        in toolchain
    )
    assert (
        """
        filter { files { "**.cpp", "**.cxx", "**.cc" } }
            buildoptions { "-stdlib=libc++", "-sMEMORY64=1", "%s" }
        filter {}
        """
        % flags
        in toolchain
    )
    assert 'linkoptions { "-sMEMORY64=1", "%s" }' % flags in toolchain



import os
import textwrap

from requests import Response

from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient, TestRequester
from conan.internal.util.files import save


conanfile = textwrap.dedent("""
    from conan import ConanFile
    from conan.tools.files import download

    class Pkg(ConanFile):
        settings = "os", "compiler"

        def source(self):
            download(self, "http://foo.bar/file", "filename.txt")
    """)


class MyHttpRequester(TestRequester):

    def get(self, _, **kwargs):
        resp = Response()
        # resp._content = b'{"results": []}'
        resp.status_code = 200
        resp._content = b''
        # This will be captured in the TestClient.out too
        print("KWARGS auth: {}".format(kwargs["auth"]))
        print("KWARGS verify: {}".format(kwargs["verify"]))
        print("KWARGS cert: {}".format(kwargs["cert"]))
        return resp


class TestClientCerts:

    def test_pic_custom_path_client_certs(self):
        folder = temp_folder()
        mycert_path = os.path.join(folder, "mycert.crt").replace("\\", '/')
        mykey_path = os.path.join(folder, "mycert.key").replace("\\", '/')
        save(mycert_path, "Fake Cert")
        save(mykey_path, "Fake Key")

        client = TestClient(requester_class=MyHttpRequester)
        conan_conf = textwrap.dedent("""
                                    core.net.http:client_cert= ("{}", "{}")
                                """.format(mycert_path, mykey_path))
        client.save_home({"global.conf": conan_conf})
        client.save({"conanfile.py": conanfile})
        client.run("create . --name=foo --version=1.0")
        assert "KWARGS cert: ('{}', '{}')".format(mycert_path, mykey_path) in client.out



import pytest

from conan.test.utils.tools import GenConanfile, TestClient


@pytest.mark.parametrize("build_mode", [None, "patch_mode"])
def test_package_id_not_affected_test_requires(build_mode):
    """
    By default, test_requires do not affect the package_id
    """
    c = TestClient()
    if build_mode is not None:
        c.save_home({"global.conf": f"core.package_id:default_build_mode={build_mode}"})
    c.save({"gtest/conanfile.py": GenConanfile("gtest", "1.0"),
            "engine/conanfile.py": GenConanfile("engine", "1.0").with_test_requires("gtest/1.0")})
    c.run("create gtest")
    c.run("create engine")
    c.run("list engine:*")
    assert "engine/1.0" in c.out
    assert "gtest" not in c.out


def test_package_id_not_affected_test_requires_transitive():
    """
    By default, transitive deps of test_requires do not affect the package_id
    """
    c = TestClient()

    c.save({"zlib/conanfile.py": GenConanfile("zlib", "1.0"),
            "gtest/conanfile.py": GenConanfile("gtest", "1.0").with_requires("zlib/1.0"),
            "engine/conanfile.py": GenConanfile("engine", "1.0").with_test_requires("gtest/1.0")})
    c.run("create zlib")
    c.run("create gtest")
    c.run("create engine")
    c.run("list engine:*")
    assert "engine/1.0" in c.out
    assert "gtest" not in c.out
    assert "zlib" not in c.out



import pytest
from unittest.mock import patch

from conan.api.input import UserInput


class TestUserInput:

    @patch("conan.api.input.UserInput.get_username", return_value="username")
    @patch("conan.api.input.UserInput.get_password", return_value="passwd")
    def test_request_login(self, m1, m2):
        user_input = UserInput(non_interactive=False)

        # Use mocked ones
        u, p = user_input.request_login(remote_name="lol")
        assert u == "username"
        assert p == "passwd"

        # Use from argument
        username = "it's me!"
        u, p = user_input.request_login(remote_name="lol", username=username)
        assert u == username
        assert p == "passwd"



import re
import textwrap

from conan.test.utils.tools import TestClient


def test_sconsdeps():
    dep = textwrap.dedent("""
        from conan import ConanFile
        class ExampleConanIntegration(ConanFile):
            name = "{dep}"
            version = "0.1"
            def package_info(self):
                self.cpp_info.includedirs = ["{dep}_includedir"]
                self.cpp_info.libdirs = ["{dep}_libdir"]
                self.cpp_info.bindirs = ["{dep}_bindir"]
                self.cpp_info.libs = ["{dep}_lib"]
                self.cpp_info.frameworks = ["{dep}_frameworks"]
                self.cpp_info.frameworkdirs = ["{dep}_frameworkdirs"]
                self.cpp_info.defines = ["{dep}_defines"]
                self.cpp_info.cxxflags = ["{dep}_cxxflags"]
                self.cpp_info.cflags = ["{dep}_cflags"]
                self.cpp_info.sharedlinkflags = ["{dep}_sharedlinkflags"]
                self.cpp_info.exelinkflags = ["{dep}_exelinkflags"]
        """)

    consumer = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.layout import basic_layout

        class ExampleConanIntegration(ConanFile):
            generators = 'SConsDeps'
            requires = 'dep1/0.1', 'dep2/0.1'
        """)

    c = TestClient()
    c.save({"dep1/conanfile.py": dep.format(dep="dep1"),
            "dep2/conanfile.py": dep.format(dep="dep2"),
            "consumer/conanfile.py": consumer})
    c.run("create dep1")
    c.run("create dep2")
    c.run("install consumer")
    sconsdeps = c.load("consumer/SConscript_conandeps")

    # remove all cache paths from the output but the last component
    def clean_paths(text):
        text = text.replace("\\", "/")
        pattern = r"'[A-Za-z]?[:]?[/]?[^']+/([^'/]+)'"
        return re.sub(pattern, r"'\1'", text)

    expected_content = ["""
        "conandeps" : {
            "CPPPATH"     : ['dep2_includedir', 'dep1_includedir'],
            "LIBPATH"     : ['dep2_libdir', 'dep1_libdir'],
            "BINPATH"     : ['dep2_bindir', 'dep1_bindir'],
            "LIBS"        : ['dep2_lib', 'dep1_lib'],
            "FRAMEWORKS"  : ['dep2_frameworks', 'dep1_frameworks'],
            "FRAMEWORKPATH" : ['dep2_frameworkdirs', 'dep1_frameworkdirs'],
            "CPPDEFINES"  : ['dep2_defines', 'dep1_defines'],
            "CXXFLAGS"    : ['dep2_cxxflags', 'dep1_cxxflags'],
            "CCFLAGS"     : ['dep2_cflags', 'dep1_cflags'],
            "SHLINKFLAGS" : ['dep2_sharedlinkflags', 'dep1_sharedlinkflags'],
            "LINKFLAGS"   : ['dep2_exelinkflags', 'dep1_exelinkflags'],
        },
        """, """
        "dep1" : {
            "CPPPATH"     : ['dep1_includedir'],
            "LIBPATH"     : ['dep1_libdir'],
            "BINPATH"     : ['dep1_bindir'],
            "LIBS"        : ['dep1_lib'],
            "FRAMEWORKS"  : ['dep1_frameworks'],
            "FRAMEWORKPATH" : ['dep1_frameworkdirs'],
            "CPPDEFINES"  : ['dep1_defines'],
            "CXXFLAGS"    : ['dep1_cxxflags'],
            "CCFLAGS"     : ['dep1_cflags'],
            "SHLINKFLAGS" : ['dep1_sharedlinkflags'],
            "LINKFLAGS"   : ['dep1_exelinkflags'],
        },
        "dep1_version" : "0.1",
        """, """
        "dep2" : {
            "CPPPATH"     : ['dep2_includedir'],
            "LIBPATH"     : ['dep2_libdir'],
            "BINPATH"     : ['dep2_bindir'],
            "LIBS"        : ['dep2_lib'],
            "FRAMEWORKS"  : ['dep2_frameworks'],
            "FRAMEWORKPATH" : ['dep2_frameworkdirs'],
            "CPPDEFINES"  : ['dep2_defines'],
            "CXXFLAGS"    : ['dep2_cxxflags'],
            "CCFLAGS"     : ['dep2_cflags'],
            "SHLINKFLAGS" : ['dep2_sharedlinkflags'],
            "LINKFLAGS"   : ['dep2_exelinkflags'],
        },
        "dep2_version" : "0.1",
        """]

    clean_sconsdeps = clean_paths(sconsdeps)
    for block in expected_content:
        assert block in clean_sconsdeps



import os
import platform
import textwrap

import pytest

from conan.test.utils.mocks import ConanFileMock
from conan.test.utils.tools import TestClient
from conan.tools.files import replace_in_file
from test.conftest import tools_locations


@pytest.mark.parametrize("arch, expected_arch", [('armv8', 'aarch64'),
                                                 ('armv7', 'arm'),
                                                 ('x86', 'i386'),
                                                 ('x86_64', 'x86_64')
                                                 ])
@pytest.mark.tool("android_ndk")
@pytest.mark.tool("autotools")
@pytest.mark.skipif(platform.system() != "Darwin", reason="NDK only installed on MAC")
def test_android_gnutoolchain_cross_compiling(arch, expected_arch):
    profile_host = textwrap.dedent("""
    include(default)

    [settings]
    os = Android
    os.api_level = 21
    arch = {arch}

    [conf]
    tools.android:ndk_path={ndk_path}
    """)
    ndk_path = tools_locations["android_ndk"]["system"]["path"][platform.system()]
    profile_host = profile_host.format(
        arch=arch,
        ndk_path=ndk_path
    )

    client = TestClient(path_with_spaces=False)
    # FIXME: Change this when we have gnu_lib as template in the new command
    client.run("new autotools_lib -d name=hello -d version=1.0")
    replace_in_file(ConanFileMock(), os.path.join(client.current_folder, "conanfile.py"),
                    "AutotoolsToolchain", "GnuToolchain")
    client.save({"profile_host": profile_host})
    client.run("build . --profile:build=default --profile:host=profile_host")
    libhello = os.path.join("build-release", "src", ".libs", "libhello.a")
    # Check binaries architecture
    client.run_command('objdump -f "%s"' % libhello)
    assert "architecture: %s" % expected_arch in client.out



import os
import platform
import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_package_from_system():
    """
    If a node declares "system_package" property, the CMakeDeps generator will skip generating
    the -config.cmake and the other files for that node but will keep the "find_dependency" for
    the nodes depending on it. That will cause that cmake looks for the config files elsewhere
    https://github.com/conan-io/conan/issues/8919"""
    client = TestClient()
    dep2 = str(GenConanfile().with_name("dep2").with_version("1.0")
               .with_settings("os", "arch", "build_type"))
    dep2 += """
    def package_info(self):
        self.cpp_info.set_property("cmake_find_mode", "None")
        self.cpp_info.set_property("cmake_file_name", "custom_dep2")

    """
    client.save({"conanfile.py": dep2})
    client.run("create .")

    dep1 = GenConanfile().with_name("dep1").with_version("1.0").with_require("dep2/1.0")\
                         .with_settings("os", "arch", "build_type")
    client.save({"conanfile.py": dep1})
    client.run("create .")

    consumer = GenConanfile().with_name("consumer").with_version("1.0").\
        with_require("dep1/1.0").with_generator("CMakeDeps").\
        with_settings("os", "arch", "build_type")
    client.save({"conanfile.py": consumer})
    client.run("install .")

    assert os.path.exists(os.path.join(client.current_folder, "dep1-config.cmake"))
    assert not os.path.exists(os.path.join(client.current_folder, "dep2-config.cmake"))
    assert not os.path.exists(os.path.join(client.current_folder, "custom_dep2-config.cmake"))
    host_arch = client.get_default_host_profile().settings['arch']
    dep1_contents = client.load(f"dep1-release-{host_arch}-data.cmake")
    assert 'list(APPEND dep1_FIND_DEPENDENCY_NAMES custom_dep2)' in dep1_contents
    assert 'set(custom_dep2_FIND_MODE "")' in dep1_contents


def test_test_package():
    client = TestClient()
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . --name=gtest --version=1.0")
    client.run("create . --name=cmake --version=1.0")

    client.save({"conanfile.py": GenConanfile().with_tool_requires("cmake/1.0").
                with_test_requires("gtest/1.0")})

    client.run("export . --name=pkg --version=1.0")

    consumer = textwrap.dedent(r"""
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "CMakeDeps"
            requires = "pkg/1.0"
        """)
    client.save({"conanfile.py": consumer})
    client.run("install . -s:b os=Windows -s:h os=Linux -s:h compiler=gcc -s:h compiler.version=7 "
               "-s:h compiler.libcxx=libstdc++11 -s:h arch=x86_64 --build=missing")
    cmake_data = client.load("pkg-release-x86_64-data.cmake")
    assert "gtest" not in cmake_data


def test_components_error():
    # https://github.com/conan-io/conan/issues/9331
    client = TestClient()

    conan_hello = textwrap.dedent("""
        import os
        from conan import ConanFile

        from conan.tools.files import save
        class Pkg(ConanFile):
            settings = "os"

            def layout(self):
                pass

            def package_info(self):
                self.cpp_info.components["say"].includedirs = ["include"]
            """)

    client.save({"conanfile.py": conan_hello})
    client.run("create . --name=hello --version=1.0")


def test_cpp_info_component_objects():
    client = TestClient()
    conan_hello = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os", "arch", "build_type"
            def package_info(self):
                self.cpp_info.components["say"].objects = ["mycomponent.o"]
            """)

    client.save({"conanfile.py": conan_hello})
    client.run("create . --name=hello --version=1.0 -s arch=x86_64 -s build_type=Release")
    client.run("install --requires=hello/1.0@ -g CMakeDeps -s arch=x86_64 -s build_type=Release")
    with open(os.path.join(client.current_folder, "hello-Target-release.cmake")) as f:
        content = f.read()
        assert """set_property(TARGET hello::say
                     APPEND PROPERTY INTERFACE_LINK_LIBRARIES
                     $<$<CONFIG:Release>:${hello_hello_say_OBJECTS_RELEASE}>
                     $<$<CONFIG:Release>:${hello_hello_say_LIBRARIES_TARGETS}>
                     )""" in content
        # If there are componets, there is not a global cpp so this is not generated
        assert "hello_OBJECTS_RELEASE" not in content
        # But the global target is linked with the targets from the components
        assert "set_property(TARGET hello::hello APPEND PROPERTY INTERFACE_LINK_LIBRARIES " \
               "hello::say)" in content

    with open(os.path.join(client.current_folder, "hello-release-x86_64-data.cmake")) as f:
        content = f.read()
        # https://github.com/conan-io/conan/issues/11862
        # Global variables
        assert 'set(hello_OBJECTS_RELEASE "${hello_PACKAGE_FOLDER_RELEASE}/mycomponent.o")' \
               in content
        # But component variables
        assert 'set(hello_hello_say_OBJECTS_RELEASE "${hello_PACKAGE_FOLDER_RELEASE}/' \
               'mycomponent.o")' in content


def test_cpp_info_component_error_aggregate():
    # https://github.com/conan-io/conan/issues/10176
    # This test was consistently failing because "VirtualRunEnv" was not doing a "copy()"
    # of cpp_info before calling "aggregate_components()", and it was destructive, removing
    # components data
    client = TestClient()
    conan_hello = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            def package_info(self):
                self.cpp_info.components["say"].includedirs = ["include"]
            """)
    consumer = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os", "compiler", "arch", "build_type"
            requires = "hello/1.0"
            generators = "VirtualRunEnv", "CMakeDeps"
            def package_info(self):
                self.cpp_info.components["chat"].requires = ["hello::say"]
        """)
    test_package = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os", "compiler", "arch", "build_type"
            generators = "VirtualRunEnv", "CMakeDeps"

            def requirements(self):
                self.requires(self.tested_reference_str)

            def test(self):
                pass
        """)

    client.save({"hello/conanfile.py": conan_hello,
                 "consumer/conanfile.py": consumer,
                 "consumer/test_package/conanfile.py": test_package})
    client.run("create hello --name=hello --version=1.0")
    client.run("create consumer --name=consumer --version=1.0")
    assert "consumer/1.0 (test package): Running test()" in client.out


def test_cmakedeps_cppinfo_complex_strings():
    client = TestClient(path_with_spaces=False)
    conanfile = textwrap.dedent(r'''
        from conan import ConanFile
        class HelloLib(ConanFile):
            def package_info(self):
                self.cpp_info.defines.append("escape=partially \"escaped\"")
                self.cpp_info.defines.append("spaces=me you")
                self.cpp_info.defines.append("foobar=bazbuz")
                self.cpp_info.defines.append("answer=42")
        ''')
    client.save({"conanfile.py": conanfile})
    client.run("export . --name=hello --version=1.0")
    client.save({"conanfile.txt": "[requires]\nhello/1.0\n"}, clean_first=True)
    client.run("install . --build=missing -g CMakeDeps")
    arch = client.get_default_host_profile().settings['arch']
    deps = client.load(f"hello-release-{arch}-data.cmake")
    assert r"escape=partially \"escaped\"" in deps
    assert r"spaces=me you" in deps
    assert r"foobar=bazbuz" in deps
    assert r"answer=42" in deps


def test_dependency_props_from_consumer():
    client = TestClient(path_with_spaces=False)
    bar = textwrap.dedent(r'''
        from conan import ConanFile
        class FooConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            def package_info(self):
                self.cpp_info.set_property("cmake_find_mode", "both")
                self.cpp_info.components["component1"].requires = []
        ''')

    foo = textwrap.dedent(r'''
        from conan import ConanFile
        from conan.tools.cmake import CMakeDeps, cmake_layout
        class FooConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            requires = "bar/1.0"
            def layout(self):
                cmake_layout(self)
            def generate(self):
                deps = CMakeDeps(self)
                {set_find_mode}
                deps.set_property("bar", "cmake_file_name", "custom_bar_file_name")
                deps.set_property("bar", "cmake_module_file_name", "custom_bar_module_file_name")
                deps.set_property("bar", "cmake_target_name", "custom_bar_target_name")
                deps.set_property("bar", "cmake_module_target_name", "custom_bar_module_target_name")
                deps.set_property("bar::component1", "cmake_target_name", "custom_bar_component_target_name")
                deps.generate()
        ''')

    set_find_mode = """
        deps.set_property("bar", "cmake_find_mode", {find_mode})
    """

    client.save({"foo.py": foo.format(set_find_mode=""), "bar.py": bar}, clean_first=True)

    if platform.system() != "Windows":
        gen_folder = os.path.join(client.current_folder, "build", "Release", "generators")
    else:
        gen_folder = os.path.join(client.current_folder, "build", "generators")

    module_file = os.path.join(gen_folder, "module-custom_bar_module_file_nameTargets.cmake")
    components_module = os.path.join(gen_folder, "custom_bar_file_name-Target-release.cmake")
    config_file = os.path.join(gen_folder, "custom_bar_file_nameTargets.cmake")

    # uses cmake_find_mode set in bar: both
    client.run("create bar.py --name=bar --version=1.0")
    client.run("install foo.py")
    assert os.path.exists(module_file)
    assert os.path.exists(config_file)
    module_content = client.load(module_file)
    assert "add_library(custom_bar_module_target_name INTERFACE IMPORTED)" in module_content
    config_content = client.load(config_file)
    assert "add_library(custom_bar_target_name INTERFACE IMPORTED)" in config_content
    components_module_content = client.load(components_module)
    assert "add_library(bar_custom_bar_component_target_name_DEPS_TARGET INTERFACE IMPORTED)" in components_module_content

    client.save({"foo.py": foo.format(set_find_mode=set_find_mode.format(find_mode="'none'")),
                 "bar.py": bar}, clean_first=True)
    client.run("create bar.py --name=bar --version=1.0")
    client.run("install foo.py")
    assert not os.path.exists(module_file)
    assert not os.path.exists(config_file)

    client.save({"foo.py": foo.format(set_find_mode=set_find_mode.format(find_mode="'module'")),
                 "bar.py": bar}, clean_first=True)
    client.run("create bar.py --name=bar --version=1.0")
    client.run("install foo.py")
    assert os.path.exists(module_file)
    assert not os.path.exists(config_file)

    client.save({"foo.py": foo.format(set_find_mode=set_find_mode.format(find_mode="'config'")),
                 "bar.py": bar}, clean_first=True)
    client.run("create bar.py --name=bar --version=1.0")
    client.run("install foo.py")
    assert not os.path.exists(module_file)
    assert os.path.exists(config_file)


def test_props_from_consumer_build_context_activated():
    client = TestClient(path_with_spaces=False)
    bar = textwrap.dedent(r'''
        from conan import ConanFile
        class FooConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            def package_info(self):
                self.cpp_info.set_property("cmake_find_mode", "both")
                self.cpp_info.components["component1"].requires = []
        ''')

    foo = textwrap.dedent(r'''
        from conan import ConanFile
        from conan.tools.cmake import CMakeDeps, cmake_layout
        class FooConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            requires = "bar/1.0"
            tool_requires = "bar/1.0"
            def layout(self):
                cmake_layout(self)
            def generate(self):
                deps = CMakeDeps(self)
                deps.build_context_activated = ["bar"]
                deps.build_context_suffix = {{"bar": "_BUILD"}}
                {set_find_mode}

                deps.set_property("bar", "cmake_file_name", "custom_bar_file_name")
                deps.set_property("bar", "cmake_module_file_name", "custom_bar_module_file_name")
                deps.set_property("bar", "cmake_target_name", "custom_bar_target_name")
                deps.set_property("bar", "cmake_module_target_name", "custom_bar_module_target_name")
                deps.set_property("bar::component1", "cmake_target_name", "custom_bar_component_target_name")

                deps.set_property("bar", "cmake_file_name", "custom_bar_build_file_name", build_context=True)
                deps.set_property("bar", "cmake_module_file_name", "custom_bar_build_module_file_name", build_context=True)
                deps.set_property("bar", "cmake_target_name", "custom_bar_build_target_name", build_context=True)
                deps.set_property("bar", "cmake_module_target_name", "custom_bar_build_module_target_name", build_context=True)
                deps.set_property("bar::component1", "cmake_target_name", "custom_bar_build_component_target_name", build_context=True)

                deps.generate()
        ''')

    set_find_mode = """
        deps.set_property("bar", "cmake_find_mode", {find_mode})
        deps.set_property("bar", "cmake_find_mode", {find_mode}, build_context=True)
    """

    client.save({"foo.py": foo.format(set_find_mode=""), "bar.py": bar}, clean_first=True)

    if platform.system() != "Windows":
        gen_folder = os.path.join(client.current_folder, "build", "Release", "generators")
    else:
        gen_folder = os.path.join(client.current_folder, "build", "generators")

    module_file = os.path.join(gen_folder, "module-custom_bar_module_file_nameTargets.cmake")
    components_module = os.path.join(gen_folder, "custom_bar_file_name-Target-release.cmake")
    config_file = os.path.join(gen_folder, "custom_bar_file_nameTargets.cmake")

    module_file_build = os.path.join(gen_folder,
                                     "module-custom_bar_build_module_file_name_BUILDTargets.cmake")
    components_module_build = os.path.join(gen_folder,
                                           "custom_bar_build_file_name_BUILD-Target-release.cmake")
    config_file_build = os.path.join(gen_folder, "custom_bar_build_file_name_BUILDTargets.cmake")

    # uses cmake_find_mode set in bar: both
    client.run("create bar.py --name=bar --version=1.0 -pr:h=default -pr:b=default")
    client.run("install foo.py --name=foo --version=1.0 -pr:h=default -pr:b=default")
    assert os.path.exists(module_file)
    assert os.path.exists(config_file)
    assert os.path.exists(module_file_build)
    assert os.path.exists(config_file_build)

    module_content = client.load(module_file)
    assert "add_library(custom_bar_module_target_name INTERFACE IMPORTED)" in module_content
    config_content = client.load(config_file)
    assert "add_library(custom_bar_target_name INTERFACE IMPORTED)" in config_content

    module_content_build = client.load(module_file_build)
    assert "add_library(custom_bar_build_module_target_name INTERFACE IMPORTED)" in module_content_build
    config_content_build = client.load(config_file_build)
    assert "add_library(custom_bar_build_target_name INTERFACE IMPORTED)" in config_content_build

    components_module_content = client.load(components_module)
    assert "add_library(bar_custom_bar_component_target_name_DEPS_TARGET INTERFACE IMPORTED)" in components_module_content

    components_module_content_build = client.load(components_module_build)
    assert "add_library(bar_BUILD_custom_bar_build_component_target_name_DEPS_TARGET INTERFACE IMPORTED)" in components_module_content_build

    client.save(
        {"foo.py": foo.format(set_find_mode=set_find_mode.format(find_mode="'none'")), "bar.py": bar},
        clean_first=True)
    client.run("create bar.py --name=bar --version=1.0 -pr:h=default -pr:b=default")
    client.run("install foo.py --name=foo --version=1.0 -pr:h=default -pr:b=default")
    assert not os.path.exists(module_file)
    assert not os.path.exists(config_file)
    assert not os.path.exists(module_file_build)
    assert not os.path.exists(config_file_build)

    client.save({"foo.py": foo.format(set_find_mode=set_find_mode.format(find_mode="'module'")),
                 "bar.py": bar}, clean_first=True)
    client.run("create bar.py --name=bar --version=1.0 -pr:h=default -pr:b=default")
    client.run("install foo.py --name=foo --version=1.0 -pr:h=default -pr:b=default")
    assert os.path.exists(module_file)
    assert not os.path.exists(config_file)
    assert os.path.exists(module_file_build)
    assert not os.path.exists(config_file_build)

    client.save({"foo.py": foo.format(set_find_mode=set_find_mode.format(find_mode="'config'")),
                 "bar.py": bar}, clean_first=True)
    client.run("create bar.py --name=bar --version=1.0 -pr:h=default -pr:b=default")
    client.run("install foo.py --name=foo --version=1.0 -pr:h=default -pr:b=default")
    assert not os.path.exists(module_file)
    assert os.path.exists(config_file)
    assert not os.path.exists(module_file_build)
    assert os.path.exists(config_file_build)

    # invalidate upstream property setting a None, will use config that's the default
    client.save({"foo.py": foo.format(set_find_mode=set_find_mode.format(find_mode="None")),
                 "bar.py": bar}, clean_first=True)
    client.run("create bar.py --name=bar --version=1.0 -pr:h=default -pr:b=default")
    client.run("install foo.py --name=foo --version=1.0 -pr:h=default -pr:b=default")
    assert not os.path.exists(module_file)
    assert os.path.exists(config_file)
    assert not os.path.exists(module_file_build)
    assert os.path.exists(config_file_build)


def test_skip_transitive_components():
    """ when a transitive depenency is skipped, because its binary is not necessary
    (shared->static), the ``components[].requires`` clause pointing to that skipped dependency
    was failing with KeyError, as the dependency info was not there
    """
    c = TestClient()
    pkg = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"
            package_type = "shared-library"
            requires = "dep/0.1"
            def package_info(self):
                self.cpp_info.components["mycomp"].requires = ["dep::dep"]
        """)

    c.save({"dep/conanfile.py": GenConanfile("dep", "0.1").with_package_type("static-library"),
            "pkg/conanfile.py": pkg,
            "consumer/conanfile.py": GenConanfile().with_settings("build_type")
                                                   .with_requires("pkg/0.1")})
    c.run("create dep")
    c.run("create pkg")
    c.run("install consumer -g CMakeDeps -v")
    c.assert_listed_binary({"dep": ("da39a3ee5e6b4b0d3255bfef95601890afd80709", "Skip")})
    # This used to error, as CMakeDeps was raising a KeyError
    assert "'CMakeDeps' calling 'generate()'" in c.out


def test_error_missing_config_build_context():
    """
    CMakeDeps was failing, not generating the zlib-config.cmake in the
    build context, for a test_package that both requires(example/1.0) and
    tool_requires(example/1.0), which depends on zlib
    # https://github.com/conan-io/conan/issues/12664
    """
    c = TestClient()
    example = textwrap.dedent("""
        import os
        from conan import ConanFile
        class Example(ConanFile):
            name = "example"
            version = "1.0"
            requires = "game/1.0"
            generators = "CMakeDeps"
            settings = "build_type"
            def build(self):
                assert os.path.exists("math-config.cmake")
                assert os.path.exists("engine-config.cmake")
                assert os.path.exists("game-config.cmake")
            """)
    c.save({"math/conanfile.py": GenConanfile("math", "1.0").with_settings("build_type"),
            "engine/conanfile.py": GenConanfile("engine", "1.0").with_settings("build_type")
                                                                .with_require("math/1.0"),
            "game/conanfile.py": GenConanfile("game", "1.0").with_settings("build_type")
                                                            .with_requires("engine/1.0"),
            "example/conanfile.py": example,
            # The example test_package contains already requires(self.tested_reference_str)
            "example/test_package/conanfile.py": GenConanfile().with_build_requires("example/1.0")
                                                               .with_test("pass")})
    c.run("create math")
    c.run("create engine")
    c.run("create game")
    # This used to crash because of the assert inside the build() method
    c.run("create example -pr:b=default -pr:h=default")
    # Now make sure we can actually build with build!=host context
    # The debug binaries are missing, so adding --build=missing
    c.run("create example -pr:b=default -pr:h=default -s:h build_type=Debug --build=missing "
          "--build=example")

    # listed as both requires and build_requires
    c.assert_listed_require({"example/1.0": "Cache"})
    c.assert_listed_require({"example/1.0": "Cache"}, build=True)


def test_using_package_module():
    """
    This crashed, because the profile "build" didn't have "build_type"
    https://github.com/conan-io/conan/issues/13209
    """
    c = TestClient()
    c.save({"conanfile.py": GenConanfile("tool", "0.1")})
    c.run("create .")

    consumer = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import CMakeDeps
        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"
            settings = "os", "compiler", "build_type", "arch"
            tool_requires = "tool/0.1"

            def generate(self):
                deps = CMakeDeps(self)
                deps.build_context_activated = ["tool"]
                deps.build_context_build_modules = ["tool"]
                deps.generate()
        """)
    c.save({"conanfile.py": consumer,
            "profile_build": "[settings]\nos=Windows"}, clean_first=True)
    c.run("create . -pr:b=profile_build")
    # it doesn't crash anymore, it used to crash
    assert "pkg/0.1: Created package" in c.out


def test_system_libs_transitivity():
    """
    https://github.com/conan-io/conan/issues/13358
    """
    c = TestClient()
    system = textwrap.dedent("""\
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "dep"
            version = "system"
            def package_info(self):
                self.cpp_info.system_libs = ["m"]
                self.cpp_info.frameworks = ["CoreFoundation"]
            """)
    header = textwrap.dedent("""
        from conan import ConanFile
        class Header(ConanFile):
            name = "header"
            version = "0.1"
            package_type = "header-library"
            requires = "dep/system"
            def package_info(self):
                self.cpp_info.system_libs = ["dl"]
                self.cpp_info.frameworks = ["CoreDriver"]
            """)
    app = textwrap.dedent("""\
        from conan import ConanFile
        class App(ConanFile):
            requires = "header/0.1"
            settings = "build_type"
            generators = "CMakeDeps"
        """)
    c.save({"dep/conanfile.py": system,
            "header/conanfile.py": header,
            "app/conanfile.py": app})
    c.run("create dep")
    c.run("create header")
    c.run("install app")
    dep = c.load("app/dep-release-data.cmake")
    assert "set(dep_SYSTEM_LIBS_RELEASE m)" in dep
    assert "set(dep_FRAMEWORKS_RELEASE CoreFoundation)" in dep
    app = c.load("app/header-release-data.cmake")
    assert "set(header_SYSTEM_LIBS_RELEASE dl)" in app
    assert "set(header_FRAMEWORKS_RELEASE CoreDriver)" in app


class TestCMakeVersionConfigCompat:
    """
    https://github.com/conan-io/conan/issues/13809
    """
    def test_cmake_version_config_compatibility(self):
        c = TestClient()
        dep = textwrap.dedent("""\
            from conan import ConanFile
            class Pkg(ConanFile):
                name = "dep"
                version = "0.1"
                def package_info(self):
                    self.cpp_info.set_property("cmake_config_version_compat", "AnyNewerVersion")
                """)

        c.save({"conanfile.py": dep})
        c.run("create .")
        c.run("install --requires=dep/0.1 -g CMakeDeps")
        dep = c.load("dep-config-version.cmake")
        expected = textwrap.dedent("""\
            if(PACKAGE_VERSION VERSION_LESS PACKAGE_FIND_VERSION)
                set(PACKAGE_VERSION_COMPATIBLE FALSE)
            else()
                set(PACKAGE_VERSION_COMPATIBLE TRUE)

                if(PACKAGE_FIND_VERSION STREQUAL PACKAGE_VERSION)
                    set(PACKAGE_VERSION_EXACT TRUE)
                endif()
            endif()""")
        assert expected in dep

    def test_cmake_version_config_compatibility_error(self):
        c = TestClient()
        dep = textwrap.dedent("""\
            from conan import ConanFile
            class Pkg(ConanFile):
                name = "dep"
                version = "0.1"
                def package_info(self):
                    self.cpp_info.set_property("cmake_config_version_compat", "Unknown")
                """)

        c.save({"conanfile.py": dep})
        c.run("create .")
        c.run("install --requires=dep/0.1 -g CMakeDeps", assert_error=True)
        assert "Unknown cmake_config_version_compat=Unknown in dep/0.1" in c.out

    def test_cmake_version_config_compatibility_consumer(self):
        c = TestClient()
        app = textwrap.dedent("""\
            from conan import ConanFile
            from conan.tools.cmake import CMakeDeps
            class Pkg(ConanFile):
                settings = "build_type"
                requires = "dep/0.1"
                def generate(self):
                    deps = CMakeDeps(self)
                    deps.set_property("dep", "cmake_config_version_compat", "AnyNewerVersion")
                    deps.generate()
                """)

        c.save({"dep/conanfile.py": GenConanfile("dep", "0.1"),
                "app/conanfile.py": app})
        c.run("create dep")
        c.run("install app")
        dep = c.load("app/dep-config-version.cmake")
        expected = textwrap.dedent("""\
            if(PACKAGE_VERSION VERSION_LESS PACKAGE_FIND_VERSION)
                set(PACKAGE_VERSION_COMPATIBLE FALSE)
            else()
                set(PACKAGE_VERSION_COMPATIBLE TRUE)

                if(PACKAGE_FIND_VERSION STREQUAL PACKAGE_VERSION)
                    set(PACKAGE_VERSION_EXACT TRUE)
                endif()
            endif()""")
        assert expected in dep


class TestSystemPackageVersion:
    def test_component_version(self):
        c = TestClient()
        dep = textwrap.dedent("""\
            from conan import ConanFile
            class Pkg(ConanFile):
                name = "dep"
                version = "system"
                def package_info(self):
                    self.cpp_info.set_property("system_package_version", "1.0")
                    self.cpp_info.components["mycomp"].set_property("component_version", "2.3")
                """)

        c.save({"conanfile.py": dep})
        c.run("create .")
        c.run("install --requires=dep/system -g CMakeDeps -g PkgConfigDeps")
        dep = c.load("dep-config-version.cmake")
        assert 'set(PACKAGE_VERSION "1.0")' in dep
        dep = c.load("dep.pc")
        assert 'Version: 1.0' in dep
        dep = c.load("dep-mycomp.pc")
        assert 'Version: 2.3' in dep

    def test_component_version_consumer(self):
        c = TestClient()
        app = textwrap.dedent("""\
            from conan import ConanFile
            from conan.tools.cmake import CMakeDeps
            class Pkg(ConanFile):
                settings = "build_type"
                requires = "dep/system"
                def generate(self):
                    deps = CMakeDeps(self)
                    deps.set_property("dep", "system_package_version", "1.0")
                    deps.generate()
                """)

        c.save({"dep/conanfile.py": GenConanfile("dep", "system"),
                "app/conanfile.py": app})
        c.run("create dep")
        c.run("install app")
        dep = c.load("app/dep-config-version.cmake")
        assert 'set(PACKAGE_VERSION "1.0")' in dep


def test_cmakedeps_set_property_overrides():
    c = TestClient()
    app = textwrap.dedent("""\
        import os
        from conan import ConanFile
        from conan.tools.cmake import CMakeDeps
        class Pkg(ConanFile):
            settings = "build_type"
            requires = "dep/0.1", "other/0.1"
            def generate(self):
                deps = CMakeDeps(self)
                # Need the absolute path inside package
                dep = self.dependencies["dep"].package_folder
                deps.set_property("dep", "cmake_build_modules", [os.path.join(dep, "my_module1")])
                deps.set_property("dep", "nosoname", True)
                deps.set_property("other::mycomp1", "nosoname", True)
                deps.generate()
            """)

    pkg_info = {"components": {"mycomp1": {"libs": ["mylib"]}}}
    c.save({"dep/conanfile.py": GenConanfile("dep", "0.1").with_package_type("shared-library"),
            "other/conanfile.py": GenConanfile("other", "0.1").with_package_type("shared-library")
                                                              .with_package_info(pkg_info, {}),
            "app/conanfile.py": app})
    c.run("create dep")
    c.run("create other")
    c.run("install app")
    dep = c.load("app/dep-release-data.cmake")
    assert 'set(dep_BUILD_MODULES_PATHS_RELEASE "${dep_PACKAGE_FOLDER_RELEASE}/my_module1")' in dep
    assert 'set(dep_NO_SONAME_MODE_RELEASE TRUE)' in dep
    other = c.load("app/other-release-data.cmake")
    assert 'set(other_other_mycomp1_NO_SONAME_MODE_RELEASE TRUE)' in other


def test_cmakedeps_set_legacy_variable_name():
    client = TestClient()
    base_conanfile = str(GenConanfile("dep", "1.0"))
    conanfile = base_conanfile + """
    def package_info(self):
        self.cpp_info.set_property("cmake_file_name", "CMakeFileName")
        self.cpp_info.set_property("cmake_find_mode", "both")
    """
    client.save({"dep/conanfile.py": conanfile})
    client.run("create dep")
    client.run("install --requires=dep/1.0 -g CMakeDeps")

    # Check that all the CMake variables are generated with the file_name
    dep_config = client.load("CMakeFileNameConfig.cmake")
    cmake_variables = ["VERSION_STRING", "INCLUDE_DIRS", "INCLUDE_DIR", "LIBRARIES", "DEFINITIONS"]
    for variable in cmake_variables:
        assert f"CMakeFileName_{variable}" in dep_config
    dep_find = client.load("FindCMakeFileName.cmake")
    for variable in cmake_variables:
        assert f"CMakeFileName_{variable}" in dep_find

    consumer_conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import CMakeDeps
        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"
            settings = "os", "build_type", "arch", "compiler"

            def requirements(self):
                self.requires("dep/1.0")

            def generate(self):
                deps = CMakeDeps(self)
                deps.set_property("dep", "cmake_additional_variables_prefixes", ["PREFIX", "prefix", "PREFIX"])
                deps.generate()
        """)

    client.save({"consumer/conanfile.py": consumer_conanfile})
    client.run("install consumer")

    dep_config = client.load(os.path.join("consumer", "CMakeFileNameConfig.cmake"))
    for variable in cmake_variables:
        assert f"CMakeFileName_{variable}" in dep_config
        assert f"PREFIX_{variable}" in dep_config
        assert f"prefix_{variable}" in dep_config

    dep_find = client.load(os.path.join("consumer", "FindCMakeFileName.cmake"))
    for variable in cmake_variables:
        assert f"CMakeFileName_{variable}" in dep_find
        assert f"PREFIX_{variable}" in dep_find
        assert f"prefix_{variable}" in dep_find
    assert "set(prefix_FOUND 1)" in dep_find
    assert "set(PREFIX_FOUND 1)" in dep_find
    assert 'set(prefix_VERSION "1.0")' in dep_find
    assert 'set(PREFIX_VERSION "1.0")' in dep_find

    conanfile = base_conanfile + """
    def package_info(self):
        self.cpp_info.set_property("cmake_file_name", "NewCMakeFileName")
        self.cpp_info.set_property("cmake_additional_variables_prefixes", ["PREFIX", "prefix", "PREFIX"])
    """
    client.save({"dep/conanfile.py": conanfile})
    client.run("create dep")
    client.run("install --requires=dep/1.0 -g CMakeDeps")

    # Check that all the CMake variables are generated with the file_name and both prefixes
    dep_config = client.load("NewCMakeFileNameConfig.cmake")
    for variable in cmake_variables:
        assert f"NewCMakeFileName_{variable}" in dep_config
        assert f"PREFIX_{variable}" in dep_config
        assert f"prefix_{variable}" in dep_config
    # Check that variables are not duplicated
    assert dep_config.count("PREFIX_VERSION") == 1


def test_different_versions():
    # https://github.com/conan-io/conan/issues/16274
    c = TestClient()
    c.save({"dep/conanfile.py": GenConanfile("dep")})
    c.run("create dep --version 1.2")
    c.run("create dep --version 2.3")
    c.run("install --requires=dep/1.2 -g CMakeDeps")
    config = c.load("dep-config.cmake")
    assert 'set(dep_VERSION_STRING "1.2")' in config
    c.run("install --requires=dep/2.3 -g CMakeDeps")
    config = c.load("dep-config.cmake")
    assert 'set(dep_VERSION_STRING "2.3")' in config


def test_using_deployer_folder():
    """
    Related to: https://github.com/conan-io/conan/issues/16543

    CMakeDeps was failing if --deployer-folder was used. The error looked like:

    conans.errors.ConanException: Error in generator 'CMakeDeps': error generating context for 'dep/1.0': mydeploy/direct_deploy/dep/include is not absolute
    """
    c = TestClient()
    profile = textwrap.dedent("""
    [settings]
    arch=x86_64
    build_type=Release
    compiler=apple-clang
    compiler.cppstd=gnu17
    compiler.libcxx=libc++
    compiler.version=15
    os=Macos
    """)
    c.save({
        "profile": profile,
        "dep/conanfile.py": GenConanfile("dep")})
    c.run("create dep --version 1.0")
    c.run("install --requires=dep/1.0 -pr profile --deployer=direct_deploy "
          "--deployer-folder=mydeploy -g CMakeDeps")
    content = c.load("dep-release-x86_64-data.cmake")
    assert ('set(dep_PACKAGE_FOLDER_RELEASE "${CMAKE_CURRENT_LIST_DIR}/mydeploy/direct_deploy/dep")'
            in content)


def test_component_name_same_package():
    """
    When the package and the component are the same the variables declared in data and linked
    to the target have to be the same.
    https://github.com/conan-io/conan/issues/9071

    This doesn't seem to have any special treatment, in fact, last FIXME looks it could be
    problematic
    """
    c = TestClient()
    dep = textwrap.dedent("""\
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "dep"
            version = "0.1"
            def package_info(self):
                self.cpp_info.components["dep"].includedirs = ["myincludes"]
            """)
    c.save({"conanfile.py": dep})
    c.run("create .")
    c.run("install --requires=dep/0.1 -g CMakeDeps -s arch=x86_64")
    cmake_data = c.load("dep-release-x86_64-data.cmake")
    assert 'set(dep_dep_dep_INCLUDE_DIRS_RELEASE ' \
           '"${dep_PACKAGE_FOLDER_RELEASE}/myincludes")' in cmake_data
    cmake_target = c.load("dep-Target-release.cmake")
    assert 'add_library(dep_dep_dep_DEPS_TARGET INTERFACE IMPORTED)' in cmake_target
    assert 'set_property(TARGET dep::dep APPEND' in cmake_target
    # FIXME: Depending on itself, this doesn't look good
    assert 'set_property(TARGET dep::dep APPEND PROPERTY ' \
           'INTERFACE_LINK_LIBRARIES dep::dep)' in cmake_target


def test_cmakedeps_set_get_property_checktype():
    c = TestClient()
    app = textwrap.dedent("""\
        from conan import ConanFile
        from conan.tools.cmake import CMakeDeps
        class Pkg(ConanFile):
            name = "app"
            version = "0.1"
            settings = "build_type"
            requires = "dep/0.1"
            def generate(self):
                deps = CMakeDeps(self)
                deps.set_property("dep", "foo", 1)
                deps.get_property("foo", self.dependencies["dep"], check_type=list)
                deps.generate()
            """)

    c.save({"dep/conanfile.py": GenConanfile("dep", "0.1"),
            "app/conanfile.py": app})
    c.run("create dep")
    c.run("create app", assert_error=True)
    assert 'The expected type for foo is "list", but "int" was found' in c.out


def test_alias_cmakedeps_set_property():
    tc = TestClient()
    tc.save({"dep/conanfile.py": textwrap.dedent("""

        from conan import ConanFile
        class Dep(ConanFile):
            name = "dep"
            version = "1.0"
            settings = "os", "compiler", "build_type", "arch"
            def package_info(self):
                self.cpp_info.components["mycomp"].set_property("cmake_target_name", "dep::mycomponent")
        """),
             "conanfile.py": textwrap.dedent("""
             from conan import ConanFile
             from conan.tools.cmake import CMakeDeps, CMake
             class Pkg(ConanFile):
                name = "pkg"
                version = "1.0"
                settings = "os", "compiler", "build_type", "arch"

                requires = "dep/1.0"

                def generate(self):
                    deps = CMakeDeps(self)
                    deps.set_property("dep", "cmake_target_aliases", ["alias", "dep::other_name"])
                    deps.set_property("dep::mycomp", "cmake_target_aliases", ["component_alias", "dep::my_aliased_component"])
                    deps.generate()
             """)})
    tc.run("create dep")
    tc.run("install .")
    targets_data = tc.load("depTargets.cmake")
    assert "add_library(dep::dep" in targets_data
    assert "add_library(alias" in targets_data
    assert "add_library(dep::other_name" in targets_data

    assert "add_library(component_alias" in targets_data
    assert "add_library(dep::my_aliased_component" in targets_data


def test_package_info_extra_variables():
    """ Test extra_variables property - This just shows that it works,
    there are tests for cmaketoolchain that check the actual behavior
    of parsing the variables"""
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"

            def package_info(self):
                self.cpp_info.set_property("cmake_extra_variables", {"FOO": 42,
                                           "BAR": 42,
                                           "CMAKE_GENERATOR_INSTANCE": "${GENERATOR_INSTANCE}/buildTools/",
                                           "CACHE_VAR_DEFAULT_DOC": {"value": "hello world",
                                                                     "cache": True, "type": "PATH"}})
    """)
    client.save({"conanfile.py": conanfile})
    client.run("create .")

    client.run("install --requires=pkg/0.1 -g CMakeDeps "
               """-c tools.cmake.cmaketoolchain:extra_variables="{'BAR': 9}" """)
    target = client.load("pkg-config.cmake")
    assert 'set(BAR' not in target
    assert 'set(CMAKE_GENERATOR_INSTANCE "${GENERATOR_INSTANCE}/buildTools/")' in target
    assert 'set(FOO 42)' in target
    assert 'set(CACHE_VAR_DEFAULT_DOC "hello world" CACHE PATH "CACHE_VAR_DEFAULT_DOC")' in target



import pytest

from conan.api.conan_api import ConanAPI
from conan.errors import ConanException


def test_profile_api():
    # It must be an absolute path
    with pytest.raises(ConanException) as e:
        ConanAPI(cache_folder="test")
    assert "cache_folder has to be an absolute path" in str(e.value)



import os

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient
from conan.internal.util.files import save


@pytest.mark.parametrize("use_pkglist", [True, False])
@pytest.mark.parametrize("output_pkglist", [True, False])
def test_cache_integrity(use_pkglist, output_pkglist):
    t = TestClient()
    t.save({"conanfile.py": GenConanfile()})
    t.run("create . --name pkg1 --version 1.0")
    t.run("create . --name pkg2 --version=2.0")
    layout = t.created_layout()
    conaninfo = os.path.join(layout.package(), "conaninfo.txt")
    save(conaninfo, "[settings]")
    t.run("create . --name pkg3 --version=3.0")
    layout = t.created_layout()
    conaninfo = os.path.join(layout.package(), "conaninfo.txt")
    save(conaninfo, "[settings]")
    t.run("create . --name pkg4 --version=4.0")
    layout = t.created_layout()
    conaninfo = os.path.join(layout.package(), "conaninfo.txt")
    save(conaninfo, "[settings]")

    if use_pkglist:
        t.run("list *:*#* -f=json", redirect_stdout="pkglist.json")
    arg = "--list=pkglist.json" if use_pkglist else "*"

    if output_pkglist:
        arg += " --format=json"

    t.run(
        f"cache check-integrity {arg}",
        assert_error=True,
        redirect_stdout="pkglist.json" if output_pkglist else None,
    )
    assert "pkg1/1.0#4d670581ccb765839f2239cc8dff8fbd: Integrity check: ok" in t.out
    assert "pkg1/1.0#4d670581ccb765839f2239cc8dff8fbd:da39a3ee5e6b4b0d3255bfef95601890afd80709" \
           "#0ba8627bd47edc3a501e8f0eb9a79e5e: Integrity check: ok" in t.out
    assert "pkg2/2.0#4d670581ccb765839f2239cc8dff8fbd:da39a3ee5e6b4b0d3255bfef95601890afd80709" \
           "#0ba8627bd47edc3a501e8f0eb9a79e5e: ERROR: \nManifest mismatch" in t.out
    assert "pkg3/3.0#4d670581ccb765839f2239cc8dff8fbd:da39a3ee5e6b4b0d3255bfef95601890afd80709" \
           "#0ba8627bd47edc3a501e8f0eb9a79e5e: ERROR: \nManifest mismatch" in t.out
    assert "pkg4/4.0#4d670581ccb765839f2239cc8dff8fbd:da39a3ee5e6b4b0d3255bfef95601890afd80709" \
           "#0ba8627bd47edc3a501e8f0eb9a79e5e: ERROR: \nManifest mismatch" in t.out

    if output_pkglist:
        t.run("remove --list=pkglist.json -c")
    else:
        t.run("remove pkg2/2.0:da39a3ee5e6b4b0d3255bfef95601890afd80709 -c")
        t.run("remove pkg3/3.0:da39a3ee5e6b4b0d3255bfef95601890afd80709 -c")
        t.run("remove pkg4/4.0:da39a3ee5e6b4b0d3255bfef95601890afd80709 -c")
    t.run("cache check-integrity *")
    assert "pkg1/1.0#4d670581ccb765839f2239cc8dff8fbd: Integrity check: ok" in t.out
    assert "pkg2/2.0#4d670581ccb765839f2239cc8dff8fbd: Integrity check: ok" in t.out
    assert "pkg3/3.0#4d670581ccb765839f2239cc8dff8fbd: Integrity check: ok" in t.out
    assert "pkg4/4.0#4d670581ccb765839f2239cc8dff8fbd: Integrity check: ok" in t.out


@pytest.mark.parametrize("output_pkglist", [True, False])
def test_cache_integrity_missing_recipe_manifest(output_pkglist):
    t = TestClient()
    t.save({"conanfile.py": GenConanfile()})
    t.run("create . --name pkg1 --version 1.0")
    t.run("create . --name pkg2 --version=2.0")
    layout = t.exported_layout()
    manifest = os.path.join(layout.export(), "conanmanifest.txt")
    os.remove(manifest)
    t.run("create . --name pkg3 --version=3.0")

    if output_pkglist:
        t.run("cache check-integrity * -f json", assert_error=True, redirect_stdout="pkglist.json")
    else:
        t.run("cache check-integrity *", assert_error=True)
    assert "pkg1/1.0#4d670581ccb765839f2239cc8dff8fbd: Integrity check: ok" in t.out
    assert "pkg2/2.0#4d670581ccb765839f2239cc8dff8fbd: ERROR: Manifest missing" in t.out
    assert "pkg3/3.0#4d670581ccb765839f2239cc8dff8fbd: Integrity check: ok" in t.out
    assert "ERROR: There are corrupted artifacts, check the error logs" in t.out

    t.run(f"remove {'--list pkglist.json' if output_pkglist else 'pkg2*'}  -c")
    t.run("cache check-integrity *")
    assert "pkg1/1.0#4d670581ccb765839f2239cc8dff8fbd:da39a3ee5e6b4b0d3255bfef95601890afd80709" \
           "#0ba8627bd47edc3a501e8f0eb9a79e5e: Integrity check: ok" in t.out
    assert "pkg3/3.0#4d670581ccb765839f2239cc8dff8fbd: Integrity check: ok" in t.out
    assert "Integrity check: ok" in t.out


@pytest.mark.parametrize("output_pkglist", [True, False])
def test_cache_integrity_missing_package_manifest(output_pkglist):
    t = TestClient()
    t.save({"conanfile.py": GenConanfile()})
    t.run("create . --name pkg1 --version 1.0")
    t.run("create . --name pkg2 --version=2.0")
    layout = t.created_layout()
    manifest = os.path.join(layout.package(), "conanmanifest.txt")
    os.remove(manifest)
    t.run("create . --name pkg3 --version=3.0")

    if output_pkglist:
        t.run("cache check-integrity * -f json", assert_error=True, redirect_stdout="pkglist.json")
    else:
        t.run("cache check-integrity *", assert_error=True)
    assert "pkg1/1.0#4d670581ccb765839f2239cc8dff8fbd: Integrity check: ok" in t.out
    assert "pkg2/2.0#4d670581ccb765839f2239cc8dff8fbd:da39a3ee5e6b4b0d3255bfef95601890afd80709" \
           "#0ba8627bd47edc3a501e8f0eb9a79e5e: ERROR: Manifest missing" in t.out
    assert "pkg3/3.0#4d670581ccb765839f2239cc8dff8fbd: Integrity check: ok" in t.out
    assert "ERROR: There are corrupted artifacts, check the error logs" in t.out

    t.run(f"remove {'--list pkglist.json' if output_pkglist else 'pkg2*'}  -c")
    t.run("cache check-integrity *")
    assert "pkg1/1.0#4d670581ccb765839f2239cc8dff8fbd:da39a3ee5e6b4b0d3255bfef95601890afd80709" \
           "#0ba8627bd47edc3a501e8f0eb9a79e5e: Integrity check: ok" in t.out
    assert "pkg3/3.0#4d670581ccb765839f2239cc8dff8fbd: Integrity check: ok" in t.out
    assert "Integrity check: ok" in t.out


@pytest.mark.parametrize("output_pkglist", [True, False])
def test_cache_integrity_missing_package_conaninfo(output_pkglist):
    t = TestClient()
    t.save({"conanfile.py": GenConanfile()})
    t.run("create . --name pkg1 --version 1.0")
    t.run("create . --name pkg2 --version=2.0")
    layout = t.created_layout()
    conaninfo = os.path.join(layout.package(), "conaninfo.txt")
    os.remove(conaninfo)

    if output_pkglist:
        t.run("cache check-integrity * -f json", assert_error=True, redirect_stdout="pkglist.json")
    else:
        t.run("cache check-integrity *", assert_error=True)
    assert "pkg1/1.0#4d670581ccb765839f2239cc8dff8fbd: Integrity check: ok" in t.out
    assert "pkg2/2.0#4d670581ccb765839f2239cc8dff8fbd:da39a3ee5e6b4b0d3255bfef95601890afd80709" \
           "#0ba8627bd47edc3a501e8f0eb9a79e5e: ERROR: \nManifest mismatch" in t.out

    t.run(f"remove {'--list pkglist.json' if output_pkglist else 'pkg2*'}  -c")
    t.run("cache check-integrity *")
    assert "pkg1/1.0#4d670581ccb765839f2239cc8dff8fbd:da39a3ee5e6b4b0d3255bfef95601890afd80709" \
           "#0ba8627bd47edc3a501e8f0eb9a79e5e: Integrity check: ok" in t.out


def test_cache_integrity_missing_package_file():
    t = TestClient()
    t.save({"conanfile.py": GenConanfile().with_package_file("myfile", "mycontents!")})
    t.run("create . --name pkg --version 1.0")
    layout = t.created_layout()
    os.remove(os.path.join(layout.package(), "myfile"))

    t.run("cache check-integrity *", assert_error=True)
    assert "pkg/1.0#2f2609c8e5c87bf836c3fdaa6096b55d:da39a3ee5e6b4b0d3255bfef95601890afd80709" \
           "#d950d0cd76f6bba62c8add9c68d1aeb3: ERROR: \nManifest mismatch" in t.out


def test_cache_integrity_export_sources():
    # https://github.com/conan-io/conan/issues/14840
    t = TestClient(default_server_user=True)
    t.save({"conanfile.py": GenConanfile("pkg", "0.1").with_exports_sources("src/*"),
            "src/mysource.cpp": ""})
    t.run("create .")
    t.run("cache check-integrity *")
    assert "pkg/0.1#ae07161b81ab07f7f1f746391668df0e: Integrity check: ok" in t.out

    # If we download, integrity should be ok
    # (it failed before, because the manifest is not complete)
    t.run("upload * -r=default -c")
    t.run("remove * -c")
    t.run("install --requires=pkg/0.1")
    t.run("cache check-integrity *")
    assert "pkg/0.1#ae07161b81ab07f7f1f746391668df0e: Integrity check: ok" in t.out



import os
import textwrap

from conan.test.utils.tools import TestClient, GenConanfile


class TestInfo:

    def _create(self, name, version, deps=None, export=True):
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class pkg(ConanFile):
                name = "{name}"
                version = "{version}"
                license = {license}
                description = "blah"
                url = "myurl"
                {requires}
            """)
        requires = ""
        if deps:
            requires = "requires = {}".format(", ".join('"{}"'.format(d) for d in deps))

        conanfile = conanfile.format(name=name, version=version, requires=requires,
                                     license='"MIT"')

        self.client.save({"conanfile.py": conanfile}, clean_first=True)
        if export:
            self.client.run("export . --user=lasote --channel=stable")

    def test_graph(self):
        self.client = TestClient()

        test_deps = {
            "hello0": ["hello1", "hello2", "hello3"],
            "hello1": ["hello4"],
            "hello2": [],
            "hello3": ["hello7"],
            "hello4": ["hello5", "hello6"],
            "hello5": [],
            "hello6": [],
            "hello7": ["hello8"],
            "hello8": ["hello9", "hello10"],
            "hello9": [],
            "hello10": [],
        }

        def create_export(testdeps, name):
            deps = testdeps[name]
            for dep in deps:
                create_export(testdeps, dep)

            expanded_deps = ["%s/0.1@lasote/stable" % dep for dep in deps]
            export = False if name == "hello0" else True
            self._create(name, "0.1", expanded_deps, export=export)

        create_export(test_deps, "hello0")

        # arbitrary case - file will be named according to argument
        self.client.run("graph info . --format=dot")
        contents = self.client.stdout

        expected = textwrap.dedent("""
            "hello8/0.1@lasote/stable" -> "hello9/0.1@lasote/stable"
            "hello8/0.1@lasote/stable" -> "hello10/0.1@lasote/stable"
            "hello4/0.1@lasote/stable" -> "hello5/0.1@lasote/stable"
            "hello4/0.1@lasote/stable" -> "hello6/0.1@lasote/stable"
            "hello3/0.1@lasote/stable" -> "hello7/0.1@lasote/stable"
            "hello7/0.1@lasote/stable" -> "hello8/0.1@lasote/stable"
            "conanfile.py (hello0/0.1)" -> "hello1/0.1@lasote/stable"
            "conanfile.py (hello0/0.1)" -> "hello2/0.1@lasote/stable"
            "conanfile.py (hello0/0.1)" -> "hello3/0.1@lasote/stable"
            "hello1/0.1@lasote/stable" -> "hello4/0.1@lasote/stable"
            """)
        for line in expected.splitlines():
            assert line in contents

    def test_graph_html(self):
        self.client = TestClient()

        test_deps = {
            "hello0": ["hello1"],
            "hello1": [],
        }

        def create_export(testdeps, name):
            deps = testdeps[name]
            for dep in deps:
                create_export(testdeps, dep)

            expanded_deps = ["%s/0.1@lasote/stable" % dep for dep in deps]
            export = False if name == "hello0" else True
            self._create(name, "0.1", expanded_deps, export=export)

        create_export(test_deps, "hello0")

        # arbitrary case - file will be named according to argument
        self.client.run("graph info . --format=html")
        html = self.client.stdout
        # Just make sure it doesn't crash
        assert "<body>" in html


def test_user_templates():
    """ Test that a user can override the builtin templates putting templates/graph.html and
    templates/graph.dot in the home
    """
    c = TestClient()
    c.save({'lib.py': GenConanfile("lib", "0.1")})
    c.run("create lib.py")
    template_folder = os.path.join(c.cache_folder, 'templates')
    c.save({"graph.html": '{{ base_template_path }}',
            "graph.dot": '{{ base_template_path }}'}, path=template_folder)
    c.run("graph info --requires=lib/0.1 --format=html")
    assert template_folder in c.stdout
    c.run("graph info --requires=lib/0.1 --format=dot")
    assert template_folder in c.stdout


def test_graph_info_html_error_reporting_output():
    tc = TestClient()
    tc.save({"lib/conanfile.py": GenConanfile("lib"),
             "ui/conanfile.py": GenConanfile("ui", "1.0").with_requirement("lib/1.0"),
             "math/conanfile.py": GenConanfile("math", "1.0").with_requirement("lib/2.0")})
    tc.run("export lib/ --version=1.0")
    tc.run("export lib/ --version=2.0")
    tc.run("export ui")
    tc.run("export math")

    tc.run("graph info --requires=math/1.0 --requires=ui/1.0 --format=html", assert_error=True,
           redirect_stdout="graph.html")
    assert "ERROR: Version conflict:" in tc.out  # check that it doesn't crash

    # change order,  just in case
    tc.run("graph info --requires=ui/1.0 --requires=math/1.0 --format=html", assert_error=True,
           redirect_stdout="graph.html")
    assert "ERROR: Version conflict:" in tc.out  # check that it doesn't crash

    # direct conflict also doesn't crash
    tc.run("graph info --requires=ui/1.0 --requires=lib/2.0 --format=html", assert_error=True,
           redirect_stdout="graph.html")
    assert "ERROR: Version conflict:" in tc.out  # check that it doesn't crash
    # Check manually
    # tc.run_command(f"{tc.current_folder}/graph.html")


def test_graph_conflict_diamond():
    c = TestClient()
    c.save({"math/conanfile.py": GenConanfile("math"),
            "engine/conanfile.py": GenConanfile("engine", "1.0").with_requires("math/1.0"),
            "ai/conanfile.py": GenConanfile("ai", "1.0").with_requires("math/1.0.1"),
            "game/conanfile.py": GenConanfile("game", "1.0").with_requires("engine/1.0", "ai/1.0"),
            })
    c.run("create math --version=1.0")
    c.run("create math --version=1.0.1")
    c.run("create math --version=1.0.2")
    c.run("create engine")
    c.run("create ai")
    c.run("graph info game --format=html", assert_error=True, redirect_stdout="graph.html")
    # check that it doesn't crash
    assert "ERROR: Version conflict: Conflict between math/1.0.1 and math/1.0 in the graph." in c.out


def test_graph_missing_error():
    c = TestClient()
    c.save({"engine/conanfile.py": GenConanfile("engine", "1.0").with_requires("math/1.0"),
            "game/conanfile.py": GenConanfile("game", "1.0").with_requires("engine/1.0"),
            })
    c.run("export engine")
    c.run("graph info game --format=html", assert_error=True, redirect_stdout="graph.html")
    # check that it doesn't crash, tested manually
    assert "ERROR: Package 'math/1.0' not resolved: No remote defined" in c.out



import glob
import os
import platform
import textwrap
from xml.dom import minidom

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


@pytest.mark.parametrize(
    "arch,exp_platform",
    [
        ("x86", "Win32"),
        ("x86_64", "x64"),
        ("armv7", "ARM"),
        ("armv8", "ARM64"),
    ],
)
def test_msbuilddeps_maps_architecture_to_platform(arch, exp_platform):
    client = TestClient()
    client.run("new msbuild_lib -d name=hello -d version=0.1")
    client.run(f"install . -g MSBuildDeps -s arch={arch} -pr:b=default")
    toolchain = client.load(os.path.join("conan", "conantoolchain.props"))
    expected_import = f"""<Import Condition="'$(Configuration)' == 'Release' And '$(Platform)' == '{exp_platform}'" Project="conantoolchain_release_{exp_platform.lower()}.props"/>"""
    assert expected_import in toolchain


@pytest.mark.parametrize(
    "build_type,arch,configuration,exp_platform",
    [
        ("Release", "x86", "Release - Test", "Win32"),
        ("Debug", "x86_64", "Debug - Test", "x64"),
    ],
)
@pytest.mark.parametrize(
    "config_key,platform_key",
    [
        ("GlobalConfiguration", "GlobalPlatform"),
        (None, None),
    ],
)
def test_msbuilddeps_import_condition(build_type, arch, configuration, exp_platform, config_key, platform_key):
    client = TestClient()
    app = textwrap.dedent(f"""
        from conan import ConanFile
        from conan.tools.microsoft import MSBuildDeps
        class App(ConanFile):
            requires = ("lib/0.1")
            settings = "arch", "build_type"
            options = {{"configuration": ["ANY"],
                       "platform": ["Win32", "x64"]}}

            def generate(self):
                ms = MSBuildDeps(self)
                ms.configuration_key = "{config_key}"
                ms.configuration = self.options.configuration
                ms.platform_key = "{platform_key}"
                ms.platform = self.options.platform
                ms.generate()
        """)

    # Remove custom set of keys to test default values
    app = app.replace(f'ms.configuration_key = "{config_key}"', "") if config_key is None else app
    app = app.replace(f'ms.platform_key = "{platform_key}"', "") if platform_key is None else app
    config_key_expected = "Configuration" if config_key is None else config_key
    platform_key_expected = "Platform" if platform_key is None else platform_key

    client.save({"app/conanfile.py": app,
                 "lib/conanfile.py": GenConanfile("lib", "0.1").with_package_type("header-library")})
    client.run("create lib")
    client.run(f'install app -s build_type={build_type} -s arch={arch} -o *:platform={exp_platform} -o *:configuration="{configuration}"')

    assert os.path.exists(os.path.join(client.current_folder, "app", "conan_lib.props"))
    dep = client.load(os.path.join(client.current_folder, "app", "conan_lib.props"))
    expected_import = f"""<Import Condition="'$({config_key_expected})' == '{configuration}' And '$({platform_key_expected})' == '{exp_platform}'" Project="conan_lib_{configuration.lower()}_{exp_platform.lower()}.props"/>"""
    assert expected_import in dep


def test_msbuilddeps_format_names():
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            name = "pkg.name-more+"
            version = "1.0"

            def package_info(self):
                self.cpp_info.components["libmpdecimal++"].libs = ["libmp++"]
                self.cpp_info.components["mycomp.some-comp+"].libs = ["mylib"]
                self.cpp_info.components["libmpdecimal++"].requires = ["mycomp.some-comp+"]
        """)
    c.save({"conanfile.py": conanfile})
    c.run("create . -s arch=x86_64")
    # Issue: https://github.com/conan-io/conan/issues/11822
    c.run("install --require=pkg.name-more+/1.0@ -g MSBuildDeps -s build_type=Release -s arch=x86_64")
    # Checking that MSBuildDeps builds correctly the XML file
    # loading all .props and xml parse them to check no errors
    pkg_more = c.load("conan_pkg_name-more_.props")
    assert "$(conan_pkg_name-more__libmpdecimal___props_imported)" in pkg_more
    assert "$(conan_pkg_name-more__mycomp_some-comp__props_imported)" in pkg_more

    some_comp = c.load("conan_pkg_name-more__mycomp_some-comp_.props")
    assert "<conan_pkg_name-more__mycomp_some-comp__props_imported>" in some_comp

    libmpdecimal = c.load("conan_pkg_name-more__libmpdecimal__.props")
    assert "<conan_pkg_name-more__libmpdecimal___props_imported>" in libmpdecimal

    libmpdecimal_release = c.load("conan_pkg_name-more__libmpdecimal___release_x64.props")
    assert "$(conan_pkg_name-more__mycomp_some-comp__props_imported)" in libmpdecimal_release

    counter = 0
    for f in os.listdir(c.current_folder):
        if f.endswith(".props"):
            content = c.load(f)
            minidom.parseString(content)
            counter += 1
    assert counter == 8


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
class TestMSBuildDepsSkips:
    # https://github.com/conan-io/conan/issues/15624
    def test_msbuilddeps_skipped_deps(self):
        c = TestClient()
        c.save({"liba/conanfile.py": GenConanfile("liba", "0.1").with_package_type("header-library"),
                "libb/conanfile.py": GenConanfile("libb", "0.1").with_package_type("static-library")
                                                                .with_requires("liba/0.1"),
                "app/conanfile.py": GenConanfile().with_requires("libb/0.1")
                                                  .with_settings("arch", "build_type")})
        c.run("create liba")
        c.run("create libb")
        c.run("install app -g MSBuildDeps")
        assert not os.path.exists(os.path.join(c.current_folder, "app", "conan_liba.props"))
        assert os.path.exists(os.path.join(c.current_folder, "app", "conan_libb.props"))
        libb = c.load("app/conan_libb.props")
        assert "conan_liba" not in libb
        libb = c.load("app/conan_libb_release_x64.props")
        assert "conan_liba" not in libb
        libb = c.load("app/conan_libb_vars_release_x64.props")
        assert "conan_liba" not in libb

    def test_msbuilddeps_skipped_deps_components(self):
        c = TestClient()
        libb = textwrap.dedent("""
            from conan import ConanFile
            class Libb(ConanFile):
                name = "libb"
                version = "0.1"
                package_type = "static-library"
                requires = "liba/0.1"
                def package_info(self):
                    self.cpp_info.components["mycomp"].libs = ["mycomplib"]
                    self.cpp_info.components["mycomp"].requires = ["liba::liba"]
            """)

        c.save({"liba/conanfile.py": GenConanfile("liba", "0.1").with_package_type("header-library"),
                "libb/conanfile.py": libb,
                "app/conanfile.py": GenConanfile().with_requires("libb/0.1")
                                                  .with_settings("arch", "build_type")})
        c.run("create liba")
        c.run("create libb")
        c.run("install app -g MSBuildDeps")
        assert not os.path.exists(os.path.join(c.current_folder, "app", "conan_liba.props"))
        assert os.path.exists(os.path.join(c.current_folder, "app", "conan_libb.props"))
        libb = c.load("app/conan_libb.props")
        assert "conan_liba" not in libb
        libb = c.load("app/conan_libb_mycomp.props")
        assert "conan_liba" not in libb
        libb = c.load("app/conan_libb_mycomp_release_x64.props")
        assert "conan_liba" not in libb
        libb = c.load("app/conan_libb_mycomp_vars_release_x64.props")
        assert "conan_liba" not in libb


@pytest.mark.skipif(platform.system() != "Windows", reason="MSBuildDeps broken with POSIX paths")
@pytest.mark.parametrize("withdepl", [False, True])
def test_msbuilddeps_relocatable(withdepl):
    c = TestClient()
    c.save({
        "libh/conanfile.py": GenConanfile("libh", "0.1").with_package_type("header-library"),
        "libs/conanfile.py": GenConanfile("libs", "0.2").with_package_type("static-library")
                                                        .with_requires("libh/0.1"),
        "libd/conanfile.py": GenConanfile("libd", "0.3").with_package_type("shared-library"),
        "app/conanfile.py": GenConanfile().with_requires("libh/0.1", "libs/0.2", "libd/0.3")
                                          .with_settings("arch", "build_type"),
    })

    c.run("create libh")
    c.run("create libs")
    c.run("create libd")
    c.run("install app -g MSBuildDeps" + (" -d full_deploy" if withdepl else ""))

    for dep in ["libh", "libs", "libd"]:
        text = c.load(f"app/conan_{dep}_vars_release_x64.props")
        marker = f"Conan{dep}RootFolder"
        value = text.split(f"<{marker}>")[1].split(f"</{marker}>")[0]
        if withdepl:
            # path should be relative, since artifacts are moved along with project
            prefix = '$(MSBuildThisFileDirectory)\\'
            assert value.startswith(prefix)
            tail = value[len(prefix):]
            assert not os.path.isabs(tail)
        else:
            # path should be absolute, since conan cache does not move with project
            assert os.path.isabs(value)
            assert '$(' not in value

    if withdepl:
        # extra checks: no absolute paths allowed anywhere in props
        propsfiles = glob.glob(os.path.join(c.current_folder, "app/*.props"))
        assert len(propsfiles) > 0
        for fn in propsfiles:
            text = c.load(fn)
            text = text.replace('\\', '/')
            folder = c.current_folder.replace('\\', '/')
            assert folder not in text


def test_msbuilddeps_consume_meson():
    c = TestClient()
    pkga = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import save
        class Pkg(ConanFile):
            name = "pkga"
            version = "0.1"
            package_type = "static-library"
            def package(self):
                save(self, os.path.join(self.package_folder, "lib", "libpkga.a"), "")
            def package_info(self):
                self.cpp_info.libs = ["pkga"]
        """)

    c.save({"pkga/conanfile.py": pkga,
            "app/conanfile.py": GenConanfile().with_requires("pkga/0.1")
                                              .with_settings("arch", "build_type")})
    c.run("create pkga -s arch=x86_64")
    c.run("install app -g MSBuildDeps -s arch=x86_64")
    deps = c.load("app/conan_pkga_vars_release_x64.props")
    assert "<ConanpkgaLibraries>libpkga.a;</ConanpkgaLibraries>" in deps



import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import NO_SETTINGS_PACKAGE_ID, TestClient
from conan.internal.util.files import save


def test_double_package_id_call():
    # https://github.com/conan-io/conan/issues/3085
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class TestConan(ConanFile):
            settings = "os", "arch"

            def package_id(self):
                self.output.info("Calling package_id()")
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=pkg --version=0.1 --user=user --channel=testing")
    out = str(client.out)
    assert 1 == out.count("pkg/0.1@user/testing: Calling package_id()")


def test_remove_option_setting():
    # https://github.com/conan-io/conan/issues/2826
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class TestConan(ConanFile):
            settings = "os"
            options = {"opt": [True, False]}
            default_options = {"opt": False}

            def package_id(self):
                self.output.info("OPTION OPT=%s" % self.info.options.opt)
                del self.info.settings.os
                del self.info.options.opt
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=pkg --version=0.1 --user=user --channel=testing -s os=Windows")
    assert "pkg/0.1@user/testing: OPTION OPT=False" in client.out
    assert "pkg/0.1@user/testing: Package '%s' created" % NO_SETTINGS_PACKAGE_ID in client.out
    client.run("create . --name=pkg --version=0.1 --user=user --channel=testing -s os=Linux -o pkg/*:opt=True")
    assert "pkg/0.1@user/testing: OPTION OPT=True" in client.out
    assert "pkg/0.1@user/testing: Package '%s' created" % NO_SETTINGS_PACKAGE_ID in client.out


def test_value_parse():
    # https://github.com/conan-io/conan/issues/2816
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class TestConan(ConanFile):
            name = "test"
            version = "0.1"
            settings = "os", "arch", "build_type"

            def package_id(self):
                self.info.settings.arch = "kk=kk"
                self.info.settings.os = "yy=yy"
        """)

    client = TestClient(default_server_user=True)
    client.save({"conanfile.py": conanfile})
    client.run("create . ")
    client.run("list *:*")
    assert "arch: kk=kk" in client.out
    client.run("upload * -r default -c")
    client.run("list *:* -r=default")
    assert "arch: kk=kk" in client.out
    client.run("remove * -c")
    client.run("install --requires=test/0.1")
    client.run("list *:*")
    assert "arch: kk=kk" in client.out


def test_option_in():
    # https://github.com/conan-io/conan/issues/7299
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class TestConan(ConanFile):
            options = {"fpic": [True, False]}
            default_options = {"fpic": True}
            def package_id(self):
                if "fpic" in self.info.options:
                    self.output.info("fpic is an option!!!")
                if "fpic" in self.info.options:  # Not documented
                    self.output.info("fpic is an info.option!!!")
                if "other" not in self.info.options:
                    self.output.info("other is not an option!!!")
                if "other" not in self.info.options:  # Not documented
                    self.output.info("other is not an info.option!!!")
                try:
                    self.options.whatever
                except Exception as e:
                    self.output.error("OPTIONS: %s" % e)
                try:
                    self.info.options.whatever
                except Exception as e:
                    self.output.error("INFO: %s" % e)

        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=pkg --version=0.1 --user=user --channel=testing")
    assert "fpic is an option!!!" in client.out
    assert "fpic is an info.option!!!" in client.out
    assert "other is not an option!!!" in client.out
    assert "other is not an info.option!!!" in client.out
    assert "OPTIONS: 'self.options' access in 'package_id()' method is forbidden" in client.out
    assert "ERROR: INFO: option 'whatever' doesn't exist" in client.out


def test_build_type_remove_windows():
    # https://github.com/conan-io/conan/issues/7603
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os", "compiler", "arch", "build_type"
            def package_id(self):
                if self.info.settings.os == "Windows" and self.info.settings.compiler == "msvc":
                   del self.info.settings.build_type
                   del self.info.settings.compiler.runtime
                   del self.info.settings.compiler.runtime_type
        """)
    client.save({"conanfile.py": conanfile})
    client.run('create . --name=pkg --version=0.1 -s os=Windows -s compiler=msvc -s arch=x86_64 '
               '-s compiler.version=190 -s build_type=Release -s compiler.runtime=dynamic')
    package_id = "6a98270da6641cc6668b83daf547d67451910cf0"
    client.assert_listed_binary({"pkg/0.1": (package_id, "Build")})
    client.run('install --requires=pkg/0.1@ -s os=Windows -s compiler=msvc -s arch=x86_64 '
               '-s compiler.version=190 -s build_type=Debug -s compiler.runtime=dynamic')
    client.assert_listed_binary({"pkg/0.1": (package_id, "Cache")})


def test_package_id_requires_info():
    """ if we dont restrict ``package_id()`` to use only ``self.info`` it will do nothing and fail
    if we ``del self.settings.arch`` instead of ``del self.info.settings.arch``
    https://github.com/conan-io/conan/issues/12693
    """
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class TestConan(ConanFile):
            settings = "os", "arch"

            def package_id(self):
                if self.info.settings.os == "Windows":
                    del self.info.settings.arch
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=pkg --version=0.1 -s os=Windows -s arch=armv8")
    client.assert_listed_binary({"pkg/0.1": ("ebec3dc6d7f6b907b3ada0c3d3cdc83613a2b715", "Build")})
    client.run("create . --name=pkg --version=0.1 -s os=Windows -s arch=x86_64")
    client.assert_listed_binary({"pkg/0.1": ("ebec3dc6d7f6b907b3ada0c3d3cdc83613a2b715", "Build")})


def test_package_id_validate_settings():
    """ ``self.info`` has no validation, as it allows to be mutated
    """
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class TestConan(ConanFile):
            settings = "os", "arch"

            def package_id(self):
                if self.info.settings.os == "DONT_EXIST":
                    del self.info.settings.arch
        """)
    c = TestClient()
    c.save({"conanfile.py": conanfile})
    c.run("create . --name=pkg --version=0.1")
    # It used to fail,


class TestBuildRequiresHeaderOnly:
    def test_header_only(self):
        c = TestClient(light=True)
        save(c.paths.global_conf_path, "core.package_id:default_build_mode=minor_mode")
        pkg = textwrap.dedent("""\
            from conan import ConanFile
            class Pkg(ConanFile):
                name = "pkg"
                version = "0.1"
                tool_requires = "tool/[*]"
                def package_id(self):
                    self.info.clear()
                """)
        c.save({"tool/conanfile.py": GenConanfile("tool"),
                "pkg/conanfile.py": pkg})
        c.run("create tool --version=1.0")
        c.run("create pkg")
        pkgid = c.created_package_id("pkg/0.1")
        c.run("create tool --version=1.2")
        c.run("install --requires=pkg/0.1")
        c.assert_listed_binary({"pkg/0.1": (pkgid, "Cache")})

    def test_header_only_implements(self):
        c = TestClient(light=True)
        save(c.paths.global_conf_path, "core.package_id:default_build_mode=minor_mode")
        pkg = textwrap.dedent("""\
            from conan import ConanFile
            class Pkg(ConanFile):
                name = "pkg"
                version = "0.1"
                tool_requires = "tool/[*]"
                package_type = "header-library"
                implements = ["auto_header_only"]
                """)
        c.save({"tool/conanfile.py": GenConanfile("tool"),
                "pkg/conanfile.py": pkg})
        c.run("create tool --version=1.0")
        c.run("create pkg")
        pkgid = c.created_package_id("pkg/0.1")
        c.run("create tool --version=1.2")
        c.run("install --requires=pkg/0.1")
        c.assert_listed_binary({"pkg/0.1": (pkgid, "Cache")})


def test_explicit_implements():
    c = TestClient()
    pkg = textwrap.dedent("""\
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"
            settings = "os"
            options = {"header_only": [True, False]}

            def package_id(self):
                if self.package_type == "header-library":
                    self.info.clear()
             """)
    c.save({"conanfile.py": pkg})
    c.run("create . -s os=Windows -o *:header_only=True")
    pkgid = c.created_package_id("pkg/0.1")
    c.run("create . -s os=Linux -o *:header_only=True")
    pkgid2 = c.created_package_id("pkg/0.1")
    assert pkgid == pkgid2

    c.run("create . -s os=Windows -o *:header_only=False")
    pkgid3 = c.created_package_id("pkg/0.1")
    assert pkgid3 != pkgid
    c.run("create . -s os=Linux -o *:header_only=False")
    pkgid4 = c.created_package_id("pkg/0.1")
    assert pkgid4 != pkgid
    assert pkgid3 != pkgid4



import textwrap

from conan.test.utils.tools import TestClient


class TestNJobs:

    def test_cpu_count_override(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.build import build_jobs

            class Conan(ConanFile):
                name = "hello0"
                version = "0.1"

                def build(self):
                    self.output.warning("CPU COUNT=> %s" % build_jobs(self))
            """)
        client.save({"conanfile.py": conanfile})
        client.run("create . -c tools.build:jobs=5")
        assert "CPU COUNT=> 5" in client.out



import json
import os
import textwrap

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


class TestPlatformRequires:
    def test_platform_requires(self):
        client = TestClient(light=True)
        client.save({"conanfile.py": GenConanfile("pkg", "1.0").with_requires("dep/1.0"),
                     "profile": "[platform_requires]\ndep/1.0"})
        client.run("create . -pr=profile")
        assert "dep/1.0 - Platform" in client.out

    def test_platform_requires_non_matching(self):
        """ if what is specified in [platform_requires] doesn't match what the recipe requires, then
        the platform_requires will not be used, and the recipe will use its declared version
        """
        client = TestClient(light=True)
        client.save({"dep/conanfile.py": GenConanfile("dep", "1.0"),
                     "conanfile.py": GenConanfile("pkg", "1.0").with_requires("dep/1.0"),
                     "profile": "[platform_requires]\ndep/1.1"})
        client.run("create dep")
        client.run("create . -pr=profile")
        assert "dep/1.0#6a99f55e933fb6feeb96df134c33af44 - Cache" in client.out

    def test_platform_requires_range(self):
        client = TestClient(light=True)
        client.save({"conanfile.py": GenConanfile("pkg", "1.0").with_requires("dep/[>=1.0]"),
                     "profile": "[platform_requires]\ndep/1.1"})
        client.run("create . -pr=profile")
        assert "dep/1.1 - Platform" in client.out

    def test_platform_requires_range_non_matching(self):
        """ if what is specified in [platform_requires] doesn't match what the recipe requires, then
        the platform_requires will not be used, and the recipe will use its declared version
        """
        client = TestClient(light=True)
        client.save({"dep/conanfile.py": GenConanfile("dep", "1.1"),
                     "conanfile.py": GenConanfile("pkg", "1.0").with_requires("dep/[>=1.0]"),
                     "profile": "[platform_requires]\ndep/0.1"})
        client.run("create dep")
        client.run("create . -pr=profile")
        assert "dep/1.1#af79f1e3973b7d174e7465038c3f5f36 - Cache" in client.out

    def test_platform_requires_no_host(self):
        """
        platform_requires must not affect tool-requires
        """
        client = TestClient(light=True)
        client.save({"conanfile.py": GenConanfile("pkg", "1.0").with_tool_requires("dep/1.0"),
                     "profile": "[platform_requires]\ndep/1.0"})
        client.run("create . -pr=profile", assert_error=True)
        assert "ERROR: Package 'dep/1.0' not resolved: No remote defined" in client.out

    def test_platform_requires_build_context(self):
        """
        platform_requires must work in the build context too
        """
        client = TestClient(light=True)
        client.save({"tool/conanfile.py": GenConanfile("tool", "1.0").with_requires("dep/1.0"),
                     "pkg/conanfile.py": GenConanfile("pkg", "1.0").with_tool_requires("tool/1.0"),
                     "profile": "[settings]\nos=Linux\n[platform_requires]\ndep/1.0"})
        client.run("create tool -pr:b=profile --build-require")
        assert "dep/1.0 - Platform" in client.out
        client.run("create pkg -pr:b=profile")
        assert "dep/1.0 - Platform" in client.out

    def test_graph_info_platform_requires_range(self):
        """
        graph info doesn't crash
        """
        client = TestClient(light=True)
        client.save({"conanfile.py": GenConanfile("pkg", "1.0").with_requires("dep/[>=1.0]"),
                     "profile": "[platform_requires]\ndep/1.1"})
        client.run("graph info . -pr=profile")
        assert "dep/1.1 - Platform" in client.out

    def test_consumer_resolved_version(self):
        client = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                requires = "dep/[>=1.0]"

                def generate(self):
                    for r, _ in self.dependencies.items():
                        self.output.info(f"DEPENDENCY {r.ref}")
                """)
        client.save({"conanfile.py": conanfile,
                     "profile": "[platform_requires]\ndep/1.1"})
        client.run("install . -pr=profile")
        assert "dep/1.1 - Platform" in client.out
        assert "conanfile.py: DEPENDENCY dep/1.1" in client.out

    def test_consumer_resolved_revision(self):
        client = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                requires = "dep/1.1"

                def generate(self):
                    for r, _ in self.dependencies.items():
                        self.output.info(f"DEPENDENCY {repr(r.ref)}")
                """)
        client.save({"conanfile.py": conanfile,
                     "profile": "[platform_requires]\ndep/1.1#rev1"})
        client.run("install . -pr=profile")
        assert "dep/1.1 - Platform" in client.out
        assert "conanfile.py: DEPENDENCY dep/1.1#rev1" in client.out

        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
               requires = "dep/1.1#rev1"

               def generate(self):
                   for r, _ in self.dependencies.items():
                       self.output.info(f"DEPENDENCY {repr(r.ref)}")
               """)
        client.save({"conanfile.py": conanfile})
        client.run("install . -pr=profile")
        assert "dep/1.1 - Platform" in client.out
        assert "conanfile.py: DEPENDENCY dep/1.1#rev1" in client.out

    def test_consumer_unresolved_revision(self):
        """ if a recipe specifies an exact revision and so does the profile
        and it doesn't match, it is an error
        """
        client = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                requires = "dep/1.1#rev2"

                def generate(self):
                    for r, _ in self.dependencies.items():
                        self.output.info(f"DEPENDENCY {repr(r.ref)}")
                """)
        client.save({"conanfile.py": conanfile,
                     "profile": "[platform_requires]\ndep/1.1#rev1"})
        client.run("install . -pr=profile", assert_error=True)
        assert "ERROR: Package 'dep/1.1' not resolved" in client.out

    def test_platform_requires_with_options(self):
        """ https://github.com/conan-io/conan/issues/15685
        """
        client = TestClient(light=True)
        client.save({"conanfile.py": GenConanfile("pkg", "1.0").with_requires("dep/1.0"),
                     "profile": "[platform_requires]\ndep/1.0"})
        client.run("create . -pr=profile -o *:myoption=True")
        assert "dep/1.0 - Platform" in client.out

        conanfile = GenConanfile("pkg", "1.0").with_requirement("dep/1.0",
                                                                options={"dep/1.0:myoption": True})
        client.save({"conanfile.py": conanfile,
                     "profile": "[platform_requires]\ndep/1.0"})
        client.run("create . -pr=profile")  # This crashed before for non-existing options
        assert "dep/1.0 - Platform" in client.out


class TestPlatformTestRequires:
    def test_platform_requires(self):
        client = TestClient(light=True)
        client.save({"conanfile.py": GenConanfile("pkg", "1.0").with_test_requires("dep/1.0"),
                     "profile": "[platform_requires]\ndep/1.0"})
        client.run("create . -pr=profile")
        assert "dep/1.0 - Platform" in client.out


class TestPlatformRequiresLock:

    def test_platform_requires_range(self):
        c = TestClient(light=True)
        c.save({"conanfile.py": GenConanfile("pkg", "1.0").with_requires("dep/[>=1.0]"),
                "profile": "[platform_requires]\ndep/1.1"})
        c.run("lock create . -pr=profile")
        assert "dep/1.1 - Platform" in c.out
        lock = json.loads(c.load("conan.lock"))
        assert lock["requires"] == ["dep/1.1"]

        c.run("install .", assert_error=True)
        assert "Package 'dep/1.1' not resolved: No remote defined" in c.out
        c.run("install . -pr=profile")
        assert "dep/1.1 - Platform" in c.out

        # if the profile points to another version it is an error, not in the lockfile
        c.save({"profile": "[platform_requires]\ndep/1.2"})
        c.run("install . -pr=profile", assert_error=True)
        assert "ERROR: Requirement 'dep/1.2' not in lockfile" in c.out


class TestGenerators:
    def test_platform_requires_range(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                settings = "build_type"
                requires = "dep/[>=1.0]"
                generators = "CMakeDeps", # "PkgConfigDeps"
            """)
        client.save({"conanfile.py": conanfile,
                     "profile": "[platform_requires]\ndep/1.1"})
        client.run("install . -pr=profile")
        assert "dep/1.1 - Platform" in client.out
        assert not os.path.exists(os.path.join(client.current_folder, "dep-config.cmake"))
        assert not os.path.exists(os.path.join(client.current_folder, "dep.pc"))


class TestPackageID:
    """ if a consumer depends on recipe revision or package_id what happens
    """

    @pytest.mark.parametrize("package_id_mode", ["recipe_revision_mode", "full_package_mode"])
    def test_package_id_modes(self, package_id_mode):
        """ this test validates that the computation of the downstream consumers package_id
        doesn't break even if it depends on fields not existing in upstream platform_requires, like revision
        or package_id
        """
        client = TestClient(light=True)
        client.save_home({"global.conf": f"core.package_id:default_unknown_mode={package_id_mode}"})
        client.save({"conanfile.py": GenConanfile("pkg", "1.0").with_requires("dep/1.0"),
                     "profile": "[platform_requires]\ndep/1.0"})
        client.run("create . -pr=profile")
        assert "dep/1.0 - Platform" in client.out

    def test_package_id_explicit_revision(self):
        """
        Changing the platform_requires revision affects consumers if package_revision_mode=recipe_revision
        """
        client = TestClient(light=True)
        client.save_home({"global.conf":
                              "core.package_id:default_unknown_mode=recipe_revision_mode"})
        client.save({"conanfile.py": GenConanfile("pkg", "1.0").with_requires("dep/1.0"),
                     "profile": "[platform_requires]\ndep/1.0#r1",
                     "profile2": "[platform_requires]\ndep/1.0#r2"})
        client.run("create . -pr=profile")
        assert "dep/1.0#r1 - Platform" in client.out
        assert "pkg/1.0#7ed9bbd2a7c3c4381438c163c93a9f21:" \
               "d9d32dd13cfc9734becc01287570721cd048ba19 - Build" in client.out

        client.run("create . -pr=profile2")
        # pkg gets a new package_id because it is a different revision
        assert "dep/1.0#r2 - Platform" in client.out
        assert "pkg/1.0#7ed9bbd2a7c3c4381438c163c93a9f21:" \
               "2f6bc9cf5015a7210181592d454f36687791a941 - Build" in client.out

    def test_package_id_full_mode(self):
        """
        platform_requires do not have settings or package_id, so it is ignored
        """
        client = TestClient()
        client.save_home({"global.conf": "core.package_id:default_unknown_mode=full_package_mode"})
        client.save({"conanfile.py": GenConanfile("pkg", "1.0").with_requires("dep/1.0"),
                     "profile": "[platform_requires]\ndep/1.0"})
        client.run("create . -pr=profile -s os=Linux")
        assert "dep/1.0 - Platform" in client.out
        assert "pkg/1.0#7ed9bbd2a7c3c4381438c163c93a9f21:" \
               "f2cfe57716d0a3320019f058edcd728d3379ab32 - Build" in client.out

        client.run("create . -pr=profile -s os=Windows")
        # pkg gets exactly same package_id, changing the settings, do not affect plaform package-id
        assert "dep/1.0 - Platform" in client.out
        assert "pkg/1.0#7ed9bbd2a7c3c4381438c163c93a9f21:" \
               "f2cfe57716d0a3320019f058edcd728d3379ab32 - Build" in client.out



import platform
import tempfile
import textwrap

import pytest

from test.conftest import tools_locations
from conan.test.utils.tools import TestClient


@pytest.mark.tool("cmake", "3.23")  # Android complains if <3.19
@pytest.mark.tool("ninja")  # so it easily works in Windows too
@pytest.mark.tool("android_ndk")
@pytest.mark.skipif(platform.system() != "Darwin", reason="NDK only installed on MAC")
def test_use_cmake_toolchain():
    """ This is the naive approach, we follow instruction from CMake in its documentation
        https://cmake.org/cmake/help/latest/manual/cmake-toolchains.7.html#cross-compiling-for-android
    """
    # Overriding the default folders, so they are in the same unit drive in Windows
    # otherwise AndroidNDK FAILS to build, it needs using the same unit drive
    c = TestClient(cache_folder=tempfile.mkdtemp(),
                   current_folder=tempfile.mkdtemp())
    c.run("new cmake_lib -d name=hello -d version=0.1")
    ndk_path = tools_locations["android_ndk"]["system"]["path"][platform.system()]
    android = textwrap.dedent(f"""
       [settings]
       os=Android
       os.api_level=23
       arch=x86_64
       compiler=clang
       compiler.version=12
       compiler.libcxx=c++_shared
       build_type=Release
       [conf]
       tools.android:ndk_path={ndk_path}
       tools.cmake.cmaketoolchain:generator=Ninja
       """)
    c.save({"android": android})
    c.run('create . --profile:host=android')
    assert "hello/0.1 (test package): Running test()" in c.out

    # Build locally
    c.run('build . --profile:host=android')
    assert "conanfile.py (hello/0.1): Running CMake.build()" in c.out
